/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include "grooves.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

GtkWidget*
create_dPurchaseDialog (struct vBuyWindowInfoStruct *pBuyStruct)
{
  GtkWidget *dPurchaseDialog = pBuyStruct->dPurchaseDialog;
  GtkWidget *dialog_vbox1 = pBuyStruct->dialog_vbox1;
  GtkWidget *title = pBuyStruct->title;
  GtkWidget *artist = pBuyStruct->artist;
  GtkWidget *album = pBuyStruct->album;
  GtkWidget *price = pBuyStruct->price;
  GtkWidget *dialog_action_area1 = pBuyStruct->dialog_action_area1;
  GtkWidget *button1 = pBuyStruct->button1;
  GtkWidget *purchase = pBuyStruct->purchase;

  dPurchaseDialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dPurchaseDialog), _("Purchase Item?"));
  gtk_window_set_position (GTK_WINDOW (dPurchaseDialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_default_size (GTK_WINDOW (dPurchaseDialog), 350, 150);
  gtk_window_set_type_hint (GTK_WINDOW (dPurchaseDialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (dPurchaseDialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  title = gtk_label_new (_("<b>Title:</b>"));
  gtk_widget_show (title);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), title, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (title), TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (title), TRUE);
  gtk_misc_set_alignment (GTK_MISC (title), 0, 0);

  artist = gtk_label_new (_("<b>Artist:</b>"));
  gtk_widget_show (artist);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), artist, TRUE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (artist), TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (artist), TRUE);
  gtk_misc_set_alignment (GTK_MISC (artist), 0, 0);

  album = gtk_label_new (_("<b>Album:</b>"));
  gtk_widget_show (album);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), album, TRUE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (album), TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (album), TRUE);
  gtk_misc_set_alignment (GTK_MISC (album), 0, 0);

  price = gtk_label_new (_("<b>Price:</b>"));
  gtk_widget_show (price);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), price, TRUE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (price), TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (price), TRUE);
  gtk_misc_set_alignment (GTK_MISC (price), 0, 0);

  dialog_action_area1 = GTK_DIALOG (dPurchaseDialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  button1 = gtk_button_new_with_mnemonic (_("Cancel"));
  gtk_widget_show (button1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dPurchaseDialog), button1, GTK_RESPONSE_REJECT);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
  gtk_signal_connect_object(GTK_OBJECT (button1), "clicked", gtk_widget_destroy, GTK_OBJECT(dPurchaseDialog));

  purchase = gtk_button_new_with_mnemonic (_("Purchase"));
  gtk_widget_show (purchase);
  gtk_dialog_add_action_widget (GTK_DIALOG (dPurchaseDialog), purchase, GTK_RESPONSE_ACCEPT);
  GTK_WIDGET_SET_FLAGS (purchase, GTK_CAN_DEFAULT);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (dPurchaseDialog, dPurchaseDialog, "dPurchaseDialog");
  GLADE_HOOKUP_OBJECT_NO_REF (dPurchaseDialog, dialog_vbox1, "dialog_vbox1");
  GLADE_HOOKUP_OBJECT (dPurchaseDialog, title, "title");
  GLADE_HOOKUP_OBJECT (dPurchaseDialog, artist, "artist");
  GLADE_HOOKUP_OBJECT (dPurchaseDialog, album, "album");
  GLADE_HOOKUP_OBJECT (dPurchaseDialog, price, "price");
  GLADE_HOOKUP_OBJECT_NO_REF (dPurchaseDialog, dialog_action_area1, "dialog_action_area1");
  GLADE_HOOKUP_OBJECT (dPurchaseDialog, button1, "button1");
  GLADE_HOOKUP_OBJECT (dPurchaseDialog, purchase, "purchase");

  gtk_widget_grab_default (button1);
  return dPurchaseDialog;
}
