/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "storelib.h"
#include "support.h"
#include "grooves_interface.h"
#include "grooves.h"

void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    fDoQuit();
}


void
on_menuPreferences_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  struct uGroovesSession *pSession = 0;
  
  fGetSession(&pSession);
  fShowSessionPreferencesWindow(pSession);
}


void
on_menuLogin_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    struct uGroovesSession *pSession = 0;

    fGetSession(&pSession);
    fChangeSessionState(pSession,
                        GROOVES_SSM_CONN);
}


void
on_menuSignup_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_menuRedownload_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_menuRedeem_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_menuGCRedeem_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_menuSong_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_menuPreview_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_menuPurchase_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

    GtkWidget *dPurchaseDialog;
    dPurchaseDialog = create_dPurchaseDialog (getPurchaseDialog());
    gtk_widget_show (dPurchaseDialog);

}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *dAboutDialog;
    dAboutDialog = create_dAboutDialog (getAboutDialog());
    gtk_widget_show (dAboutDialog);
}


void
on_searchButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    struct uGroovesSession *pSession = 0;

    fGetSession(&pSession);
    fAttemptSearch(pSession);
    
}


void
on_treeview1_row_activated             (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{

}


gboolean
on_treeview1_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}

void
close_about_window                     (GtkWidget       *widget,
                                        gpointer        user_data)
{

}

void closewidget(GtkWidget *widget)
{
  gtk_widget_destroy(widget);
}

void fDoLoginWindowOk(GtkWidget *widget, gpointer user_data)
{
  GtkWidget *pUNEntry = rMainStruct._loginwin.username_entry;
  GtkWidget *pPWEntry = rMainStruct._loginwin.password_entry;
  struct uGroovesSession *pSession = 0;

  if(fGetSession(&pSession) < 0)
  {
    closewidget(widget);
    return;
  }
  
  if(pSession->_pUserName != 0)
    free(pSession->_pUserName);

  if(pSession->_pPassWord != 0)
    free(pSession->_pPassWord);

  pSession->_pUserName = malloc(1+strlen(
			         gtk_entry_get_text(GTK_ENTRY(pUNEntry))));

  strcpy(pSession->_pUserName,
	 gtk_entry_get_text(GTK_ENTRY(pUNEntry)));

  
  pSession->_pPassWord = malloc(1+strlen(
			         gtk_entry_get_text(GTK_ENTRY(pPWEntry))));

  strcpy(pSession->_pPassWord,
	 gtk_entry_get_text(GTK_ENTRY(pPWEntry)));

  closewidget(widget);

  fChangeSessionState(pSession,
                      GROOVES_SSM_CONN);
}

void fDoPrefsWindowApply(GtkWidget *widget, gpointer user_data)
{
  struct uGroovesSession *pSession = 0;
  GtkWidget *pSDEntry = rMainStruct._prefswin.entry1;
  GtkWidget *pBUEntry = rMainStruct._prefswin.entry2;
  GtkWidget *pALCheck = rMainStruct._prefswin.checkbutton1;
  GtkWidget *pRPCheck = rMainStruct._prefswin.checkbutton2;
  
  struct GroovesPrefBlob gpb = {0,0,0,0};

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pALCheck)) == TRUE)
    gpb._autologin = 1;
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pRPCheck)) == TRUE)
    gpb._rememberpw = 1;

  if(gtk_entry_get_text(GTK_ENTRY(pSDEntry)))
  {
    gpb._pSongsDir = gtk_entry_get_text(GTK_ENTRY(pSDEntry));
  } 

  if(gtk_entry_get_text(GTK_ENTRY(pBUEntry)))
  {
    gpb._pBaseURL = gtk_entry_get_text(GTK_ENTRY(pBUEntry));
  } 

  if(fGetSession(&pSession) < 0)
  {
    DEBUG_PRINT(("::error getting session!!"));
    return;
  }

  fApplyNewSessionSettings(pSession,
			   gpb,
			   0);
  
  fWriteSessionPrefs(pSession); //write them out :D
}

void fDoPrefsWindowOK(GtkWidget *widget, gpointer user_data)
{
  fDoPrefsWindowApply(widget, user_data);
  closewidget(widget);
}
