/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "storelib.h"
#include "grooves_interface.h"
#include "grooves.h"

char *fGetConnectionStateName(int state)
{
    switch(state)
    {
    case GROOVES_CSM_IDLE:
        return "Idle";
    case GROOVES_CSM_LINKING:
        return "Acquisition";
    case GROOVES_CSM_AUTHITS:
        return "Authentication";
    case GROOVES_CSM_CONNECTED:
        return "Connected";
    case GROOVES_CSM_TERMINATED:
        return "Terminating";
    default:
        return "Unknown";
    }
}

int fProcessLinkLine(struct uGroovesConnection *pConnection,
                     char *pOptsLine)
{
    char *pConnOptName = 0;
    char *pConnOptValue = 0;

    fStripWhitespace(pOptsLine);

    if(*pOptsLine == "#")
        return 0;

    pConnOptName = strtok(pOptsLine, "=");

    if(!strncmp("incacceptlanguage", pConnOptName, 17))
    {
        pConnection->_flags = 
            pConnection->_flags | HTTP_CONST_TRANS_INC_ACCLAN;
    } 
    else if(!strncmp("incxappltz", pConnOptName, 10))
    {
        pConnection->_flags = 
            pConnection->_flags | HTTP_CONST_TRANS_INC_TIMEZ;
    }
    else if(!strncmp("incuseragent", pConnOptName, 12))
    {
        pConnection->_flags = 
            pConnection->_flags | HTTP_CONST_TRANS_INC_UAGEN;
    }
    else if(!strncmp("incdsid", pConnOptName, 7))
    {
        DEBUG_PRINT(("::connection opts - DSID ignored\n"));
    }
    else if(!strncmp("incxvalid", pConnOptName, 9))
    {
        DEBUG_PRINT(("::connection opts - X-Validator ignored\n"));
        pConnection->_flags = 
          pConnection->_flags | HTTP_CONST_TRANS_INC_XVALID;
    }
    else if(!strncmp("incacceptenc", pConnOptName, 12))
    {
        pConnection->_flags = 
            pConnection->_flags | HTTP_CONST_TRANS_INC_ACCENC;
    }
    else if(!strncmp("incstorefront", pConnOptName, 13))
    {
        pConnection->_flags = 
            pConnection->_flags | HTTP_CONST_TRANS_INC_STOREFRONT;
    }
    else if(!strncmp("inchost", pConnOptName, 7))
    {
        pConnection->_flags = 
            pConnection->_flags | HTTP_CONST_TRANS_INC_HOST;
    } else if (!strncmp("init_url", pConnOptName, 7)){
        pConnOptValue = strtok(NULL, "=");

        pConnection->_pUrlToFollow = 
            (char *) malloc(strlen(pConnOptValue) + 1);
        
        if(!pConnection->_pUrlToFollow)
            return -1;

        bzero(pConnection->_pUrlToFollow, strlen(pConnOptValue) + 1);
        strcpy(pConnection->_pUrlToFollow, pConnOptValue);
    } else if (!strncmp("searchUrlKey", pConnOptName, 12)) {
        pConnOptValue = strtok(NULL, "=");

        pConnection->_urlBag._pSearchKey = 
            (char *) malloc(strlen(pConnOptValue) + 1);
        
        if(!pConnection->_urlBag._pSearchKey)
            return -1;

        bzero(pConnection->_urlBag._pSearchKey, strlen(pConnOptValue) + 1);
        strcpy(pConnection->_urlBag._pSearchKey, pConnOptValue);
    }
    
    return 0;
}

int fAttemptConnectionLink(struct uGroovesConnection *pConnection,
                           int nextState)
{
    struct uGroovesSession *pSession = pConnection->_pRelatedSession;
    int result = 0;
    int count = 0;
    int iFile = 0;
    char *pLinkBuf = 0;
    char *linkUrl = 0;
    char *pTempFile = 0;
    char aBufLine[256] = {0};
    
    if(!pSession)
        return -1;

    linkUrl = pSession->_currentPreferences._pBaseURL;
    
    if(!linkUrl)
        return -1;

/*  For now do a 1-shot dealie. I'll go to filebased later. Why?
 *  Should the user for some reason be unable to contact the awesome rc url
 *  I still want them to be able to fall back on the backup that's saved in
 *  their home directory. A 1-shot deal is nice, but ultimately, not good.
 */

    /*Do a 1-shot HTTP request*/
    result = fGetBufFromURL(HTTP_CONST_GETURL_ONESHOT,
                            linkUrl,
                            &pLinkBuf,
                            HTTP_CONST_TRANS_INC_ACCENC |
                            HTTP_CONST_TRANS_INC_UAGEN | 
                            HTTP_CONST_TRANS_INC_HOST);
    
    pConnection->_status = GROOVES_CSM_LINKING;

    while(fGetNextLineFromBuf(pLinkBuf, aBufLine, &count, 256))
    {
        if((aBufLine[0] == 0) ||
           (aBufLine[0] == '\r') ||
           (aBufLine[0] == '\n'))
            continue;
        
        result = fProcessLinkLine(pConnection, aBufLine);

        /*this indicates a corrupt link file*/
        if(result < 0)
            break;
    }

    free(pLinkBuf);

    if(result < 0)
        return -1;

    /*contrary to popular belief, logging in doesn't require
     *username and password. However, they're good things to have
     */
    return fChangeConnectionState(pConnection,
                                  GROOVES_CSM_CONNECTED);

}

/*Check states to figure out what to do...*/
int fProcSetupConnBuf(struct uGroovesConnection *pConnection,
                      char *pConnBuf)
{
    int result = 0;
    int httpHeaderOffset = 0;
    char aUncompressedBuf[8192] = {0};
    int searchIdx = 0;
    /*no need to do a sm check here...we won't get here on invalid state*/

    if(!pConnBuf)
    {
        DEBUG_PRINT(("ERROR! No Connection Buffer!\n"));
        return -1;
    }

#ifdef USE_ZLIB_FOR_COMPRESSED
    httpHeaderOffset = fOffsetOfHttpHeaders(pConnBuf);

    result = fDoUncompressBuffer(aUncompressedBuf, 
                                 pConnBuf + httpHeaderOffset, 
                                 8192);

    if(result < 0)
    {
        return -1;
    }
#else
    strncpy(aUncompressedBuf, pConnBuf, 8192);
#endif

    searchIdx = fGetStrIndexOf(
			       aUncompressedBuf,
			       pConnection->_urlBag._pSearchKey, 
			       0),

    result = fGetStrBetween(&pConnection->_urlBag._pSearchUrl,
                            aUncompressedBuf,
			    searchIdx,
                            "<string>",
                            "</string>");

    searchIdx = fGetStrIndexOf(
                               aUncompressedBuf,
                               "storeFront",
                               0);

    result = fGetStrBetween(&pConnection->_urlBag._pStoreFrontUrl,
                            aUncompressedBuf,
			    searchIdx,
                            "<string>",
                            "</string>");

/*    result = fDoHTTPTransaction(pConnection->_connectionSocket, 
 */                              
    
    pConnection->_status = GROOVES_CSM_CONNECTED;
    return 0;
}

int fDoConnectionConnect(struct uGroovesConnection *pConnection)
{
    struct sockaddr_in localAddr, servAddr;
    char *pUrlBuf = 0;
    int result = 0;

    GROOVES_ASSERT(pConnection, "No connection when going to connect");
    
    result = fGetSockForURL(pConnection->_pUrlToFollow,
                            &servAddr);

    if(result < 0)
        return fChangeConnectionState(pConnection, GROOVES_CSM_TERMINATED);

    pConnection->_connectionSocket = result;

    localAddr.sin_family = AF_INET;
    localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    localAddr.sin_port = htons(0);

//    servAddr.sin_port = (short)htons(80);

    result = connect(pConnection->_connectionSocket,
                     (struct sockaddr *) &servAddr, 
                     sizeof(servAddr));
    
    if(result < 0)
        return fChangeConnectionState(pConnection, GROOVES_CSM_TERMINATED);

    result = fDoHTTPTransaction(pConnection->_connectionSocket, 
                                pConnection->_pUrlToFollow,
                                &pUrlBuf,
                                (pConnection->_flags & HTTP_CONST_TRANS_INC_ACCENC) ? pConnection->_flags ^ HTTP_CONST_TRANS_INC_ACCENC : pConnection->_flags,
                                pConnection);

    if(result < 0)
        return fChangeConnectionState(pConnection, GROOVES_CSM_TERMINATED);
    

    result = fProcSetupConnBuf(pConnection, pUrlBuf);
    free(pUrlBuf);
    return result;
}

int fAttemptConnectionConn(struct uGroovesConnection *pConnection,
                           int nextState)
{
    int result = 0;

    GROOVES_ASSERT(pConnection, "Null connection when attempting to open");
    
    if((pConnection->_status != GROOVES_CSM_LINKING) &&
       (pConnection->_status != GROOVES_CSM_AUTHITS))
    {
        DEBUG_PRINT(("::fAttemptConnectionConn() Open in invalid state[%s]",
                     fGetConnectionStateName(pConnection->_status)));
        return -1;
    }
    
    result = fDoConnectionConnect(pConnection);
    
    if(result < 0)
    {
        return -1;
    }
    
    pConnection->_status = GROOVES_CSM_CONNECTED;

    return 0;

}

int fAttemptConnectionTerm(struct uGroovesConnection *pConnection,
                           int nextState)
{
    int result = 0;
    GROOVES_ASSERT(pConnection, "No Connection when moving to TERM state");
    
    //delete the url to follow, set the flags to 0, close the connectionsocket,
    //clear out the url bag, and set the status to term. Then go to idle state

    if(pConnection->_connectionSocket)
        close(pConnection->_connectionSocket);

    pConnection->_connectionSocket = 0;

    if(pConnection->_pUrlToFollow)
        free(pConnection->_pUrlToFollow);

    pConnection->_pUrlToFollow = 0;

    if(pConnection->_urlBag._pSearchUrl)
        free(pConnection->_urlBag._pSearchUrl);

    pConnection->_urlBag._pSearchUrl = 0;

    if(pConnection->_urlBag._pSearchKey)
        free(pConnection->_urlBag._pSearchKey);

    pConnection->_urlBag._pSearchKey = 0;

    pConnection->_flags = 0;

    pConnection->_status = GROOVES_CSM_TERMINATED;

    fChangeSessionState(pConnection->_pRelatedSession,
                        GROOVES_SSM_NOCONN);

    return fChangeConnectionState(pConnection, GROOVES_CSM_IDLE);
}

int fChangeConnectionState(struct uGroovesConnection *pConnection,
                           int nextState)
{

    DEBUG_PRINT(("::fChangeConnectionState() from [%s] to [%s]\n",
                 fGetConnectionStateName(pConnection->_status),
                 fGetConnectionStateName(nextState)));

    switch(nextState)
    {
    case GROOVES_CSM_IDLE:
        /*nothing to do when going to idle state
         *except set status, and return 0
         */
        pConnection->_status = GROOVES_CSM_IDLE;
        return 0;
    case GROOVES_CSM_LINKING:
        return fAttemptConnectionLink(pConnection,
                                      nextState);
    case GROOVES_CSM_CONNECTED:
        return fAttemptConnectionConn(pConnection,
                                      nextState);
    case GROOVES_CSM_AUTHITS:
        return 0;
/*        return fAttemptConnectionAuth(pConnection,
                                      nextState);
*/
    case GROOVES_CSM_TERMINATED:
        return fAttemptConnectionTerm(pConnection,
                                      nextState);
    default:
        return -1;
    }
}

int fDoConnSearch(struct uGroovesConnection *pConnection)
{
    int result = 0;
    char searchUrl[512] = {0};
    char *pSearchTerms = 
        gtk_entry_get_text(GTK_ENTRY(rMainStruct.entry1));

    if(!pSearchTerms)
        return -1;

    snprintf(searchUrl,
             512,
             "%s?term=%s",
             pConnection->_urlBag._pSearchUrl,
             pSearchTerms);
    
    result = fDoHTTPTransaction(pConnection->_connectionSocket,
                                searchUrl,
                                &pSearchTerms,
                                pConnection->_flags,
                                pConnection);

    if(result < 0)
        fChangeConnectionState(pConnection, GROOVES_CSM_TERMINATED);

    DEBUG_PRINT(("[%s]\n", pSearchTerms));

    return 0;
}

int fAttemptConnSearch(struct uGroovesConnection *pConnection)
{
    if(pConnection->_status != GROOVES_CSM_CONNECTED)
    {
        return -1;
    }

    return fDoConnSearch(pConnection);
}
