/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "grooves.h"
#include "storelib.h"
#include "preferences.h"

#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <stdlib.h>
#include <errno.h>

char *pHomeDir = 0;
char *pHomebaseAddr = "aconole.brad-x.com";
char aPrefGetStr[256] = {0};

int initFromEnv()
{
    int sizeHomeDirStr = 0;

    if(pHomeDir != 0)
    {
        /*already initialized somewhere else*/
        return 0;
    }

    if(getenv("HOME"))
    {
        sizeHomeDirStr = strlen(getenv("HOME"));
        pHomeDir = malloc(sizeHomeDirStr +1);
        strcpy(pHomeDir, getenv("HOME"));
    }
    else
    {
        printf("NO HOME DIR!\n");
        exit(-1);
    }
    
    return 0;
}

int doCheckPrefsLocations()
{
    char *groovesDir = 0;

    fGetPrefsDir(&groovesDir);

    if( mkdir(groovesDir, 0777) < 0)
    {
      switch(errno)
      {
      case EEXIST:
	break;
      default:
        printf("Error making grooves dir: [%s]:[%d]\n",
               groovesDir, errno);
      }
    }

    return 0;
}

int getLatestStoresFromHomebase()
{
    char *pDownLoc = 0;
    char aStoreURL[256] = {0};
    int result = 0;
    printf("Getting latest stores...");
    
    result = snprintf(
        aStoreURL,
        256,
        "%s://%s/%s",
        DEFPROTO,
        DEFHOST,
        STORE);

    if (result < 0)
    {
        printf("Error! Couldn't create URL\n");
        return -1;
    }
/*
    pDownLoc = (char *)malloc(strlen(getCurrentPref(GROOVES)) +
                              strlen(STORE) + 2);
    if(pDownLoc == 0)
    {
        printf("::malloc() error!\n");
        return -1;
    }

    result = snprintf(
        pDownLoc,

        strlen(getCurrentPref(GROOVES)) +
        strlen(STORE) + 2,

        "%s/%s",
        getCurrentPref(GROOVES),
        STORE);

    if(result < 0)
    {
        printf("::snprintf() error!\n");
        free(pDownLoc);
        return -1;
    }
    
    result = fGetURLToFile(aStoreURL, pDownLoc);
    if(result < 0)
    {
        printf("::fgeturltofile() error!\n");
        free(pDownLoc);
        return -1;
    }

    printf("exiting getstore\n");
    free (pDownLoc);
*/
    return -1;
}

int fGetPrefsDir(char **ppDirectoryBuf)
{
    int i = 0;

    

    i = snprintf(aPrefGetStr,
                     256,
                     "%s/%s",
                     pHomeDir,
                     ".grooves");

    *ppDirectoryBuf = aPrefGetStr;
    
    return i;

}

int fGetPrefsFile(char **ppFilenameBuf)
{
    int i = snprintf(aPrefGetStr,
                     256,
                     "%s/%s/prefs",
                     pHomeDir,
                     ".grooves");

    *ppFilenameBuf = aPrefGetStr;
    
    return i;
                     
}

int fGetTempFile(char **ppFilenameBuf, char *pTempFilename)
{
    int i = snprintf(aPrefGetStr,
                     256,
                     "%s/%s/%s",
                     pHomeDir,
                     ".grooves",
                     pTempFilename);
    
    *ppFilenameBuf = aPrefGetStr;
    
    return i;
}

int fLoadSessionPreferences(struct uGroovesSession *pSession)
{
    int iFile = 0;
    char *pPrefsFile = 0;
    char aLineBuf[256] = {0};
    int result = 0;

    initFromEnv();

    result = doCheckPrefsLocations();
    if(result < 0)
    {
        return -1;
    }

    result = fGetPrefsFile(&pPrefsFile);

    if (result < 0)
    {
        return -1;
    }
    
    iFile = fOpenFileForReading(pPrefsFile, 0);
    
    if (iFile < 0)
    {
        return -1;
    }

    while(fReadLineBufFromFile(iFile, aLineBuf, 256) > 0)
    {
        if((aLineBuf[0] == 0) ||
           (aLineBuf[0] == '\r') ||
           (aLineBuf[0] == '\n'))
            continue;
        result = fProcessSessionPref(pSession, aLineBuf);
        if(result < 0)
            break;
    }
    
    return 0;
}

int fProcessSessionPref(struct uGroovesSession *pSession, char *prefsLine)
{
    char *pPrefName = 0;
    char *pPrefValue = 0;

    fStripWhitespace(prefsLine);

    if(*prefsLine == "#")
        return 0;

    pPrefName = strtok(prefsLine, "=");

    if(!strncmp("autologin", pPrefName, 9))
    {
        pPrefValue = strtok(NULL, "=");
        pSession->_currentPreferences._autologin = atoi(pPrefValue);
    } 
    else if(!strncmp("rememberpw", pPrefName, 10))
    {
        pPrefValue = strtok(NULL, "=");
        pSession->_currentPreferences._rememberpw = atoi(pPrefValue);
    }
    else if(!strncmp("songdir", pPrefName, 7))
    {
        pPrefValue = strtok(NULL, "=");
        pSession->_currentPreferences._pSongsDir = malloc(strlen(pPrefValue)+1);
        if(!pSession->_currentPreferences._pSongsDir)
            return -1;

        bzero(pSession->_currentPreferences._pSongsDir, strlen(pPrefValue)+1);
        strcpy(pSession->_currentPreferences._pSongsDir,
               pPrefValue);
    }
    else if(!strncmp("baseurl", pPrefName, 7))
    {
        pPrefValue = strtok(NULL, "=");
        pSession->_currentPreferences._pBaseURL = malloc(strlen(pPrefValue)+1);
	if(!pSession->_currentPreferences._pBaseURL)
            return -1;
        bzero(pSession->_currentPreferences._pBaseURL, strlen(pPrefValue)+1);
	strcpy(pSession->_currentPreferences._pBaseURL,
               pPrefValue);
    }
    else if(!strncmp("username", pPrefName, 8))
    {
        pPrefValue = strtok(NULL, "=");
        pSession->_pUserName = malloc(strlen(pPrefValue)+1);
        if(!pSession->_pUserName)
            return -1;
 
        bzero(pSession->_pUserName, strlen(pPrefValue)+1);
       strcpy(pSession->_pUserName,
              pPrefValue);
    }
    else if(!strncmp("password", pPrefName, 8))
    {
        pPrefValue = strtok(NULL, "=");
        pSession->_pPassWord = malloc(strlen(pPrefValue)+1);
        if(!pSession->_pPassWord)
            return -1;
 
        bzero(pSession->_pPassWord, strlen(pPrefValue)+1);
        strcpy(pSession->_pPassWord,
               pPrefValue);
    }

    return 0;
}

int fQuit()
{
  gtk_main_quit();
  return 0;
}

int fVarPrintForPrefs(char *pArray, int sz, char *pKey, char *pValue)
{
  return snprintf(pArray,
		  sz,
		  "%s=%s\n",
		  pKey,
		  pValue);
}

int fWritePrefsToFile(struct uGroovesSession *pSession, char *pFilename)
{
  int result = 0;
  char *pFinalBuf = 0;
  char numBuf[10] = {0};
  char aPrefsWriteBuf[256] = {0};

  /*first, construct autologin*/
  snprintf(numBuf, 10, "%d", pSession->_currentPreferences._autologin);
  fVarPrintForPrefs(aPrefsWriteBuf, 256, "autologin", 
		    numBuf);
  
  pFinalBuf = malloc(strlen(aPrefsWriteBuf)+1);
  bzero(pFinalBuf, strlen(aPrefsWriteBuf));
  strcat(pFinalBuf, aPrefsWriteBuf);

  /*remember password?*/
  snprintf(numBuf, 10, "%d", pSession->_currentPreferences._rememberpw);
  fVarPrintForPrefs(aPrefsWriteBuf, 256, "rememberpw", 
		    numBuf);
  pFinalBuf = realloc(pFinalBuf, strlen(pFinalBuf)+strlen(aPrefsWriteBuf)+1);
  strcat(pFinalBuf, aPrefsWriteBuf);

  if(pSession->_currentPreferences._pSongsDir)
  {
    fVarPrintForPrefs(aPrefsWriteBuf, 256, "songdir", 
		      pSession->_currentPreferences._pSongsDir);
    pFinalBuf = realloc(pFinalBuf, strlen(pFinalBuf)+strlen(aPrefsWriteBuf)+1);
    strcat(pFinalBuf, aPrefsWriteBuf);
  }

  if(pSession->_currentPreferences._pBaseURL)
  {
    fVarPrintForPrefs(aPrefsWriteBuf, 256, "baseurl", 
		      pSession->_currentPreferences._pBaseURL);
    pFinalBuf = realloc(pFinalBuf, strlen(pFinalBuf)+strlen(aPrefsWriteBuf)+1);
    strcat(pFinalBuf, aPrefsWriteBuf);
  }

  /*Username*/
  if(pSession->_pUserName)
  {
    fVarPrintForPrefs(aPrefsWriteBuf, 256, "username", 
		      pSession->_pUserName);
    pFinalBuf = realloc(pFinalBuf, strlen(pFinalBuf)+strlen(aPrefsWriteBuf)+1);
    strcat(pFinalBuf, aPrefsWriteBuf);
  }

  if((pSession->_currentPreferences._rememberpw) &&
     (pSession->_pPassWord))
  {
    fVarPrintForPrefs(aPrefsWriteBuf, 256, "password", 
		      pSession->_pPassWord);
    pFinalBuf = realloc(pFinalBuf, strlen(pFinalBuf)+strlen(aPrefsWriteBuf)+1);
    strcat(pFinalBuf, aPrefsWriteBuf);
  }
  

  result = fWriteBufToFile(pFilename, pFinalBuf, strlen(pFinalBuf));
  
  if(result < 0)
  {
    DEBUG_PRINT(("::fWritePrefsToFile() write to file [%s][%d]:\n[%s]\n",
		 pFilename, result, pFinalBuf));
  }
  
  free(pFinalBuf);
  
  return result;
}

int fApplyNewSessionSettings(struct uGroovesSession *pSession,
			     struct GroovesPrefBlob rBlob,
			     int do_reset_conn)
{
  //first, set the new settings...
  if(rBlob._pSongsDir)
  {
    if(pSession->_currentPreferences._pSongsDir)
      free(pSession->_currentPreferences._pSongsDir);
    pSession->_currentPreferences._pSongsDir = 
              malloc(strlen(rBlob._pSongsDir));

    memcpy(pSession->_currentPreferences._pSongsDir,
	   rBlob._pSongsDir,
	   strlen(rBlob._pSongsDir));
  }

  if(rBlob._pBaseURL)
  {
    if( pSession->_currentPreferences._pBaseURL)
      free(pSession->_currentPreferences._pBaseURL);

    pSession->_currentPreferences._pBaseURL = 
               malloc(strlen(rBlob._pBaseURL));

    memcpy(pSession->_currentPreferences._pBaseURL,
	   rBlob._pBaseURL,
	   strlen(rBlob._pBaseURL));
  }

  pSession->_currentPreferences._autologin = rBlob._autologin;
  pSession->_currentPreferences._rememberpw = rBlob._rememberpw;

  /*close and then re-open the connection*/
  if(do_reset_conn)
  {
    fChangeSessionState(pSession, GROOVES_SSM_NOCONN);
    fChangeSessionState(pSession, GROOVES_SSM_CONN);
  }

  return 0;
}
