/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>

#include "interface.h"
#include "support.h"
#include "grooves.h"
#include "grooves_interface.h"

#include "storelib.h"

struct vMainWindowInfoStruct rMainStruct;

int grooves_main(int argc, char *argv[])
{
    struct uGroovesSession *pSession = 0;

    int result = 0;

/*GTK RELATED STUFF!!! DO THIS FIRST!!!*/
#ifdef ENABLE_NLS
    bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
    textdomain (GETTEXT_PACKAGE);
#endif  
    
    gtk_set_locale ();
    gtk_init (&argc, &argv);
  
    add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");
/*==================================================================*/

    result = fGetSession(&pSession);
  
    if(result < 0)
    {
        fprintf(stderr, "ERROR! Couldn't get grooves session!\n");
        return -1;
    }

    result = fInitSessionToIdle(pSession);

    if(result < 0)
    {
        fprintf(stderr, "ERROR! Grooves session couldn't be initialized\n");
        return -1;
    }

    result = fChangeSessionState(pSession, GROOVES_SSM_INIT);

    if (result < 0)
    {
        fprintf(stderr, "ERROR CHANGING SESSION STATE!\n");
        return -1;
    }

    /*DO NOT ATTEMPT TO USE / GET SESSION AFTER THIS POINT. SSM WILL FREE IT*/

    gtk_main ();

    return 0;
}

#if 0
int
gtk_grooves_main (int argc, char *argv[])
{
  GtkWidget *wMainWindow;
  
  GtkWidget *dAlertDialog;

  GtkWidget *dRedeemDialog;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  initFromEnv();

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  wMainWindow = create_wMainWindow (&rMainStruct);
  doStartTreeOff(rMainStruct);
  doGetLatestStores();
  g_signal_connect( wMainWindow, "delete_event", fDoQuit,
                    NULL ); //dirty


  gtk_widget_show_all (wMainWindow);

  gtk_main ();
  return 0;
}
#endif /*0*/

int testAssert()
{
    GROOVES_ASSERT(0, "FAIL!\n");
    return 0;
}

int main(int argc, char *argv[])
{
//    return test_Validator_code();
    return grooves_main(argc, argv);
//    return testAssert();
//    return fGetURLToFile("http://www.google.com/index.html", "/home/aconole/google.index.html");
//    return test_URL_section_get();
//    return test_base64_convert();
}
