/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include "grooves.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)


int doStartTreeOff(struct vMainWindowInfoStruct vMainBlock)
{
    GtkWidget *treeview1 = vMainBlock.treeview1;
    GtkCellRenderer *renderer;
    GtkListStore *pTestStore;
    GtkTreeIter  iter;
    GtkTreeViewColumn *pColumn = 0;
    int i = 0;
    
    renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(
      GTK_TREE_VIEW(treeview1),
      -1,
      "Title",
      renderer,
      "text", COL_TITLE,
      NULL);
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(
      GTK_TREE_VIEW(treeview1),
      -1,
      "Artist",
      renderer,
      "text", COL_ARTIST,
      NULL);
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(
      GTK_TREE_VIEW(treeview1),
      -1,
      "Album",
      renderer,
      "text", COL_ALBUM,
      NULL);
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(
      GTK_TREE_VIEW(treeview1),
      -1,
      "Time",
      renderer,
      "text", COL_TIME,
      NULL);
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(
      GTK_TREE_VIEW(treeview1),
      -1,
      "Price",
      renderer,
      "text", COL_PRICE,
      NULL);

  pTestStore = gtk_list_store_new(NUM_COLS, G_TYPE_STRING, G_TYPE_STRING,
                                  G_TYPE_STRING, G_TYPE_STRING, 
                                  G_TYPE_STRING);

  gtk_tree_view_set_model ( GTK_TREE_VIEW(treeview1), GTK_TREE_MODEL(pTestStore));
  
  gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(treeview1), TRUE);

  for(i = 0; i < NUM_COLS; i++)
  {
      pColumn = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview1), i);
      gtk_tree_view_column_set_resizable(pColumn, TRUE);
  }

  return 0;
}

GtkWidget*
create_wMainWindow (struct vMainWindowInfoStruct *pMainBlock)
{
  GtkWidget *wMainWindow = pMainBlock->wMainWindow;
  GtkWidget *vbox2 = pMainBlock->vbox2;
  GtkWidget *alignment1 = pMainBlock->alignment1;
  GtkWidget *menubar1 = pMainBlock->menubar1;
  GtkWidget *menuFile = pMainBlock->menuFile;
  GtkWidget *menuFile_menu = pMainBlock->menuFile_menu;
  GtkWidget *menuQuit = pMainBlock->menuQuit;
  GtkWidget *menuEdit = pMainBlock->menuEdit;
  GtkWidget *menuEdit_menu = pMainBlock->menuEdit_menu;
  GtkWidget *menuPreferences = pMainBlock->menuPreferences;
  GtkWidget *menuitem3 = pMainBlock->menuitem3;
  GtkWidget *menuAccount = pMainBlock->menuAccount;
  GtkWidget *menuAccount_menu = pMainBlock->menuAccount_menu;
  GtkWidget *menuLogin = pMainBlock->menuLogin;
  GtkWidget *menuSignup = pMainBlock->menuSignup;
  GtkWidget *separator1 = pMainBlock->separator1;
  GtkWidget *menuRedownload = pMainBlock->menuRedownload;
  GtkWidget *separator2 = pMainBlock->separator2;
  GtkWidget *menuRedeem = pMainBlock->menuRedeem;
  GtkWidget *menuGCRedeem = pMainBlock->menuGCRedeem;
  GtkWidget *menuSong = pMainBlock->menuSong;
  GtkWidget *menuSong_menu = pMainBlock->menuSong_menu;
  GtkWidget *menuPreview = pMainBlock->menuPreview;
  GtkWidget *menuPurchase = pMainBlock->menuPurchase;
  GtkWidget *menuitem4 = pMainBlock->menuitem4;
  GtkWidget *menuitem4_menu = pMainBlock->menuitem4_menu;
  GtkWidget *about1 = pMainBlock->about1;
  GtkWidget *vpaned1 = pMainBlock->vpaned1;
  GtkWidget *hbox2 = pMainBlock->hbox2;
  GtkWidget *entry1 = pMainBlock->entry1;
  GtkWidget *searchButton = pMainBlock->searchButton;
  GtkWidget *vbox3 = pMainBlock->vbox3;
  GtkWidget *eventbox1 = pMainBlock->eventbox1;
  GtkWidget *scrolledwindow1 = pMainBlock->scrolledwindow1;
  GtkWidget *treeview1 = pMainBlock->treeview1;
  GtkWidget *statusbar1 = pMainBlock->statusbar1;
  GtkAccelGroup *accel_group = pMainBlock->accel_group;

  accel_group = gtk_accel_group_new ();

  wMainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (wMainWindow), _("GTK+ iTunesStore"));
  gtk_window_set_default_size (GTK_WINDOW (wMainWindow), 150, 75);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (wMainWindow), vbox2);

  alignment1 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment1);
  gtk_box_pack_start (GTK_BOX (vbox2), alignment1, FALSE, FALSE, 0);

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_show (menubar1);
  gtk_container_add (GTK_CONTAINER (alignment1), menubar1);

  menuFile = gtk_menu_item_new_with_mnemonic (_("_File"));
  gtk_widget_show (menuFile);
  gtk_container_add (GTK_CONTAINER (menubar1), menuFile);

  menuFile_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuFile), menuFile_menu);

  menuQuit = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
  gtk_widget_show (menuQuit);
  gtk_container_add (GTK_CONTAINER (menuFile_menu), menuQuit);

  menuEdit = gtk_menu_item_new_with_mnemonic (_("_Edit"));
  gtk_widget_show (menuEdit);
  gtk_container_add (GTK_CONTAINER (menubar1), menuEdit);

  menuEdit_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuEdit), menuEdit_menu);

  menuPreferences = gtk_image_menu_item_new_from_stock ("gtk-preferences", accel_group);
  gtk_widget_show (menuPreferences);
  gtk_container_add (GTK_CONTAINER (menuEdit_menu), menuPreferences);

  menuitem3 = gtk_menu_item_new_with_mnemonic (_("_Country"));
  gtk_widget_show (menuitem3);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem3);

  menuAccount = gtk_menu_item_new_with_mnemonic (_("_Account"));
  gtk_widget_show (menuAccount);
  gtk_container_add (GTK_CONTAINER (menubar1), menuAccount);

  menuAccount_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuAccount), menuAccount_menu);

  menuLogin = gtk_menu_item_new_with_mnemonic (_("_Login"));
  gtk_widget_show (menuLogin);
  gtk_container_add (GTK_CONTAINER (menuAccount_menu), menuLogin);

  menuSignup = gtk_menu_item_new_with_mnemonic (_("_Signup"));
  gtk_widget_show (menuSignup);
  gtk_container_add (GTK_CONTAINER (menuAccount_menu), menuSignup);

  separator1 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (menuAccount_menu), separator1);
  gtk_widget_set_sensitive (separator1, FALSE);

  menuRedownload = gtk_menu_item_new_with_mnemonic (_("Redownload"));
  gtk_widget_show (menuRedownload);
  gtk_container_add (GTK_CONTAINER (menuAccount_menu), menuRedownload);

  separator2 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator2);
  gtk_container_add (GTK_CONTAINER (menuAccount_menu), separator2);
  gtk_widget_set_sensitive (separator2, FALSE);

  menuRedeem = gtk_menu_item_new_with_mnemonic (_("Redeem Pepsi Cap"));
  gtk_widget_show (menuRedeem);
  gtk_container_add (GTK_CONTAINER (menuAccount_menu), menuRedeem);

  menuGCRedeem = gtk_menu_item_new_with_mnemonic (_("Redeem Gift Certificate"));
  gtk_widget_show (menuGCRedeem);
  gtk_container_add (GTK_CONTAINER (menuAccount_menu), menuGCRedeem);

  menuSong = gtk_menu_item_new_with_mnemonic (_("S_ong"));
  gtk_widget_show (menuSong);
  gtk_container_add (GTK_CONTAINER (menubar1), menuSong);

  menuSong_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuSong), menuSong_menu);

  menuPreview = gtk_menu_item_new_with_mnemonic (_("Pr_eview"));
  gtk_widget_show (menuPreview);
  gtk_container_add (GTK_CONTAINER (menuSong_menu), menuPreview);

  menuPurchase = gtk_menu_item_new_with_mnemonic (_("_Buy"));
  gtk_widget_show (menuPurchase);
  gtk_container_add (GTK_CONTAINER (menuSong_menu), menuPurchase);

  menuitem4 = gtk_menu_item_new_with_mnemonic (_("_Help"));
  gtk_widget_show (menuitem4);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem4);

  menuitem4_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem4), menuitem4_menu);

  about1 = gtk_menu_item_new_with_mnemonic (_("_About"));
  gtk_widget_show (about1);
  gtk_container_add (GTK_CONTAINER (menuitem4_menu), about1);

  vpaned1 = gtk_vpaned_new ();
  gtk_widget_show (vpaned1);
  gtk_box_pack_start (GTK_BOX (vbox2), vpaned1, TRUE, TRUE, 0);
  gtk_paned_set_position (GTK_PANED (vpaned1), 36);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_paned_pack1 (GTK_PANED (vpaned1), hbox2, FALSE, TRUE);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox2), entry1, TRUE, TRUE, 0);

  searchButton = gtk_button_new_with_mnemonic (_("Search"));
  gtk_widget_show (searchButton);
  gtk_box_pack_start (GTK_BOX (hbox2), searchButton, FALSE, FALSE, 0);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_paned_pack2 (GTK_PANED (vpaned1), vbox3, TRUE, TRUE);

  eventbox1 = gtk_event_box_new ();
  gtk_widget_show (eventbox1);
  gtk_box_pack_start (GTK_BOX (vbox3), eventbox1, TRUE, TRUE, 0);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (eventbox1), scrolledwindow1);

  treeview1 = gtk_tree_view_new ();
  pMainBlock->treeview1 = treeview1;
  
  gtk_widget_show (treeview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), treeview1);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox3), statusbar1, FALSE, FALSE, 0);

  g_signal_connect ((gpointer) menuQuit, "activate",
                    G_CALLBACK (on_quit1_activate),
                    NULL);
  g_signal_connect ((gpointer) menuPreferences, "activate",
                    G_CALLBACK (on_menuPreferences_activate),
                    NULL);
  g_signal_connect ((gpointer) menuLogin, "activate",
                    G_CALLBACK (on_menuLogin_activate),
                    NULL);
  g_signal_connect ((gpointer) menuSignup, "activate",
                    G_CALLBACK (on_menuSignup_activate),
                    NULL);
  g_signal_connect ((gpointer) menuRedownload, "activate",
                    G_CALLBACK (on_menuRedownload_activate),
                    NULL);
  g_signal_connect ((gpointer) menuRedeem, "activate",
                    G_CALLBACK (on_menuRedeem_activate),
                    NULL);
  g_signal_connect ((gpointer) menuGCRedeem, "activate",
                    G_CALLBACK (on_menuGCRedeem_activate),
                    NULL);
  g_signal_connect ((gpointer) menuSong, "activate",
                    G_CALLBACK (on_menuSong_activate),
                    NULL);
  g_signal_connect ((gpointer) menuPreview, "activate",
                    G_CALLBACK (on_menuPreview_activate),
                    NULL);
  g_signal_connect ((gpointer) menuPurchase, "activate",
                    G_CALLBACK (on_menuPurchase_activate),
                    NULL);
  g_signal_connect ((gpointer) about1, "activate",
                    G_CALLBACK (on_about1_activate),
                    NULL);
  g_signal_connect ((gpointer) searchButton, "clicked",
                    G_CALLBACK (on_searchButton_clicked),
                    NULL);
  g_signal_connect ((gpointer) treeview1, "row_activated",
                    G_CALLBACK (on_treeview1_row_activated),
                    NULL);
  g_signal_connect ((gpointer) treeview1, "button_press_event",
                    G_CALLBACK (on_treeview1_button_press_event),
                    NULL);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (wMainWindow, wMainWindow, "wMainWindow");
  GLADE_HOOKUP_OBJECT (wMainWindow, vbox2, "vbox2");
  GLADE_HOOKUP_OBJECT (wMainWindow, alignment1, "alignment1");
  GLADE_HOOKUP_OBJECT (wMainWindow, menubar1, "menubar1");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuFile, "menuFile");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuFile_menu, "menuFile_menu");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuQuit, "menuQuit");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuEdit, "menuEdit");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuEdit_menu, "menuEdit_menu");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuPreferences, "menuPreferences");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuitem3, "menuitem3");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuAccount, "menuAccount");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuAccount_menu, "menuAccount_menu");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuLogin, "menuLogin");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuSignup, "menuSignup");
  GLADE_HOOKUP_OBJECT (wMainWindow, separator1, "separator1");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuRedownload, "menuRedownload");
  GLADE_HOOKUP_OBJECT (wMainWindow, separator2, "separator2");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuRedeem, "menuRedeem");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuGCRedeem, "menuGCRedeem");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuSong, "menuSong");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuSong_menu, "menuSong_menu");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuPreview, "menuPreview");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuPurchase, "menuPurchase");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuitem4, "menuitem4");
  GLADE_HOOKUP_OBJECT (wMainWindow, menuitem4_menu, "menuitem4_menu");
  GLADE_HOOKUP_OBJECT (wMainWindow, about1, "about1");
  GLADE_HOOKUP_OBJECT (wMainWindow, vpaned1, "vpaned1");
  GLADE_HOOKUP_OBJECT (wMainWindow, hbox2, "hbox2");
  GLADE_HOOKUP_OBJECT (wMainWindow, entry1, "entry1");
  GLADE_HOOKUP_OBJECT (wMainWindow, searchButton, "searchButton");
  GLADE_HOOKUP_OBJECT (wMainWindow, vbox3, "vbox3");
  GLADE_HOOKUP_OBJECT (wMainWindow, eventbox1, "eventbox1");
  GLADE_HOOKUP_OBJECT (wMainWindow, scrolledwindow1, "scrolledwindow1");
  GLADE_HOOKUP_OBJECT (wMainWindow, treeview1, "treeview1");
  GLADE_HOOKUP_OBJECT (wMainWindow, statusbar1, "statusbar1");

  gtk_window_add_accel_group (GTK_WINDOW (wMainWindow), accel_group);

  /*  g_signal_connect( wMainWindow, "delete_event", ,
      NULL ); */

  pMainBlock->entry1 = entry1;  

  return wMainWindow;
}
