/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include "grooves.h"

#include "callbacks.h"
#include "support.h"
#include "preferences.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

GtkWidget*
create_dPrefsDialog (struct vPrefsWindowInfoStruct *pPrefsWindow)
{
  GtkWidget *dPrefsDialog = 0;
  GtkWidget *dialog_vbox1 = 0;
  GtkWidget *vbox1 = 0;
  GtkWidget *vbox2 = 0;
  GtkWidget *hbox1 = 0;
  GtkWidget *songdirlabel = 0;
  GtkWidget *entry1 = 0;
  GtkWidget *entry2 = 0;
  GtkWidget *hbox2 = 0;
  GtkWidget *prefslabel2 = 0;
  GtkWidget *entry3 = 0;
  GtkWidget *vbox3 = 0;
  GtkWidget *hseparator1 = 0;
  GtkWidget *checkbutton1 = 0;
  GtkWidget *checkbutton2 = 0;
  GtkWidget *dialog_action_area4 = 0;
  GtkWidget *cancelbutton2 = 0;
  GtkWidget *applybutton1 = 0;
  GtkWidget *okbutton3 = 0;

  dPrefsDialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dPrefsDialog), _("Grooves Preferences"));

  dialog_vbox1 = GTK_DIALOG (dPrefsDialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

  songdirlabel = gtk_label_new (_("Song Directory: "));
  gtk_widget_show (songdirlabel);
  gtk_box_pack_start (GTK_BOX (hbox1), songdirlabel, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (songdirlabel), GTK_JUSTIFY_LEFT);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox1), entry1, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, TRUE, TRUE, 0);

  prefslabel2 = gtk_label_new (_("Base URL: "));
  gtk_widget_show (prefslabel2);
  gtk_box_pack_start (GTK_BOX (hbox2), prefslabel2, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (prefslabel2), GTK_JUSTIFY_LEFT);

  entry2 = gtk_entry_new ();
  gtk_widget_show (entry2);
  gtk_box_pack_start (GTK_BOX (hbox2), entry2, TRUE, TRUE, 0);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox3, TRUE, TRUE, 0);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox3), hseparator1, TRUE, TRUE, 0);

  checkbutton1 = gtk_check_button_new_with_mnemonic (_("Auto Login on start"));
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton1, FALSE, FALSE, 0);

  checkbutton2 = gtk_check_button_new_with_mnemonic (_("Remember Password"));
  gtk_widget_show (checkbutton2);
  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton2, FALSE, FALSE, 0);

  dialog_action_area4 = GTK_DIALOG (dPrefsDialog)->action_area;
  gtk_widget_show (dialog_action_area4);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area4), GTK_BUTTONBOX_END);

  cancelbutton2 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton2);
  gtk_dialog_add_action_widget (GTK_DIALOG (dPrefsDialog), cancelbutton2, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton2, GTK_CAN_DEFAULT);
  gtk_signal_connect_object(GTK_OBJECT (cancelbutton2), "clicked", gtk_widget_destroy, GTK_OBJECT(dPrefsDialog));


  applybutton1 = gtk_button_new_from_stock ("gtk-apply");
  gtk_widget_show (applybutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dPrefsDialog), applybutton1, GTK_RESPONSE_APPLY);
  GTK_WIDGET_SET_FLAGS (applybutton1, GTK_CAN_DEFAULT);
  gtk_signal_connect_object(GTK_OBJECT (applybutton1), "clicked", fDoPrefsWindowApply, GTK_OBJECT(dPrefsDialog));

  okbutton3 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton3);
  gtk_dialog_add_action_widget (GTK_DIALOG (dPrefsDialog), okbutton3, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton3, GTK_CAN_DEFAULT);
  gtk_signal_connect_object(GTK_OBJECT (okbutton3), "clicked", fDoPrefsWindowOK, GTK_OBJECT(dPrefsDialog));

  /* Store pointers to all widgets, for use by lookup_widget(). */
  /*
  GLADE_HOOKUP_OBJECT_NO_REF (dPrefsDialog, dPrefsDialog, "dPrefsDialog");
  GLADE_HOOKUP_OBJECT_NO_REF (dPrefsDialog, dialog_vbox1, "dialog_vbox1");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, vbox1, "vbox1");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, vbox2, "vbox2");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, hbox1, "hbox1");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, songdirlabel, "songdirlabel");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, entry1, "entry1");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, hbox2, "hbox2");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, prefslabel2, "prefslabel2");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, entry2, "entry2");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, vbox3, "vbox3");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, hseparator1, "hseparator1");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, checkbutton1, "checkbutton1");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, checkbutton2, "checkbutton2");
  GLADE_HOOKUP_OBJECT_NO_REF (dPrefsDialog, dialog_action_area4, "dialog_action_area4");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, cancelbutton2, "cancelbutton2");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, applybutton1, "applybutton1");
  GLADE_HOOKUP_OBJECT (dPrefsDialog, okbutton1, "okbutton3");
  */

  pPrefsWindow->dPrefsDialog = dPrefsDialog;
  pPrefsWindow->dialog_vbox1 = dialog_vbox1;
  pPrefsWindow->vbox1 = vbox1;
  pPrefsWindow->vbox2 = vbox2;
  pPrefsWindow->hbox1 = hbox1;
  pPrefsWindow->songdirlabel = songdirlabel;
  pPrefsWindow->entry1 = entry1;
  pPrefsWindow->entry2 = entry2;
  pPrefsWindow->hbox2 = hbox2;
  pPrefsWindow->prefslabel2 = prefslabel2;
  pPrefsWindow->entry3 = entry3;
  pPrefsWindow->vbox3 = vbox3;
  pPrefsWindow->hseparator1 = hseparator1;
  pPrefsWindow->checkbutton1 = checkbutton1;
  pPrefsWindow->checkbutton2 = checkbutton2;
  pPrefsWindow->dialog_action_area4 = dialog_action_area4;
  pPrefsWindow->cancelbutton2 = cancelbutton2;
  pPrefsWindow->applybutton1 = applybutton1;
  pPrefsWindow->okbutton3 = okbutton3;

  return dPrefsDialog;
}
