/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "storelib.h"
#include "grooves.h"

struct uGroovesSession *pMasterSession = 0;

char *fGetSessionStateName(int state)
{
    switch(state)
    {
    case GROOVES_SSM_IDLE:
        return "SSM_Idle";
    case GROOVES_SSM_INIT:
        return "SSM_Init";
    case GROOVES_SSM_NOCONN:
        return "SSM_NoConn";
    case GROOVES_SSM_CONN:
        return "SSM_Conn";
    case GROOVES_SSM_TERM:
        return "SSM_Term";
    default:
        return "Uknown";
    }
}

int fChangeSessionToIdle(struct uGroovesSession *pSession,
                        int nextState)
{
    //we just return to idle.
    return fInitSessionToIdle(pSession);
}

int fChangeSessionToInit(struct uGroovesSession *pSession,
                        int nextState)
{
    int result = 0;

    //we only come to here from IDLE state...any other state we have an issue
    GROOVES_ASSERT(pSession->_status == GROOVES_SSM_IDLE,
                   "Transition to session Idle state from invalid state");

    /*if this wants autologin, 
     *then it will set the pSession's 
     *attemptlogin flag
     */
    result = fLoadSessionPreferences(pSession);

    if(result < 0)
    {
        DEBUG_PRINT(("::fChangeSessionToInit() - load prefs failure[%d]\n",
                     result));
        
        return -1;
    }

    DEBUG_PRINT((":: :: Session Loaded [%d] [%d] [%s]\n",
                 pSession->_currentPreferences._autologin,
                 pSession->_currentPreferences._rememberpw,
                 pSession->_currentPreferences._pSongsDir));

    fShowMainWindow(pSession);
    
    pSession->_status = GROOVES_SSM_INIT;

    if(pSession->_currentPreferences._autologin)
        fChangeSessionState(pSession, GROOVES_SSM_CONN);
     else
        fChangeSessionState(pSession, GROOVES_SSM_NOCONN);

    return 0;
}

int fChangeSessionToNoConnFromConn(struct uGroovesSession *pSession)
{
    
    //connection state machine has taken care of connection.
    //we should just set ourselves to not having a connection
    pSession->_status = GROOVES_SSM_NOCONN;

    return 0;
    
}

int fChangeSessionToNoConn(struct uGroovesSession *pSession,
                        int nextState)
{
    GROOVES_ASSERT(pSession, "::fChangeSessionToNoConn called with no session\n");

    switch(pSession->_status)
    {
    case GROOVES_SSM_INIT:
      //from init state == do nothing.
      return 0;
    case GROOVES_SSM_CONN:
        return fChangeSessionToNoConnFromConn(pSession);
    default:
        DEBUG_PRINT(("::fChangeSessionToNoConn() - Invalid state [%s]\n",
                     fGetSessionStateName(pSession->_status)));
        return -1;
    }
}

int fChangeSessionToConn(struct uGroovesSession *pSession,
                        int nextState)
{
    int result = fAttemptSessionLogin(pSession);
    if(result == -2)
        return fChangeSessionState(pSession,
                                   GROOVES_SSM_NOCONN);
    else if(result < 0)
        return -1;
    
    pSession->_status = GROOVES_SSM_CONN;
    return 0;    
}

int fChangeSessionToTerm(struct uGroovesSession *pSession,
                        int nextState)
{
    return -1;
}


int fChangeSessionState(struct uGroovesSession *pSession,
                        int nextState)
{

    DEBUG_PRINT(("::fChangeSessionState() from [%s] to [%s]\n",
                 fGetSessionStateName(pSession->_status),
                 fGetSessionStateName(nextState)));

  /*If we're already in a state, no need to re-transition*/
    if(nextState == pSession->_status)
      return 0;

    switch(nextState)
    {
    case GROOVES_SSM_IDLE:
        return fChangeSessionToIdle(pSession, nextState);
    case GROOVES_SSM_INIT:
        return fChangeSessionToInit(pSession, nextState);
    case GROOVES_SSM_NOCONN:
        return fChangeSessionToNoConn(pSession, nextState);
    case GROOVES_SSM_CONN:
        return fChangeSessionToConn(pSession, nextState);
    case GROOVES_SSM_TERM:
        return fChangeSessionToTerm(pSession, nextState);
    default:
        DEBUG_PRINT(("Unknown state: %d\n", nextState));
        return -1;
    }
}
int fGetSession(struct uGroovesSession **ppSession)
{
    if(!pMasterSession)
    {
        pMasterSession = 
            (struct uGroovesSession *)malloc(
                sizeof(struct uGroovesSession));
    }

    *ppSession = pMasterSession;

    GROOVES_ASSERT(pMasterSession, "No Default Session!\n");

    return 0;
}

int fInitSessionToIdle(struct uGroovesSession *pCurrentSession)
{
    pCurrentSession->_pUserName = 0;
    pCurrentSession->_pPassWord = 0;
    pCurrentSession->_status    = GROOVES_SSM_IDLE;

    pCurrentSession->_currentConnection._status = GROOVES_CSM_IDLE;
    pCurrentSession->_currentConnection._connectionSocket = 0;
    pCurrentSession->_currentConnection._pRelatedSession = pCurrentSession;

    
    pCurrentSession->_currentConnection._urlBag._pSearchUrl = 0;
    pCurrentSession->_currentConnection._urlBag._pAdvSearchUrl = 0;
    pCurrentSession->_currentConnection._urlBag._pSongMetaDataUrl = 0;

    pCurrentSession->_currentConnection._urlBag._pSearchKey = 0;
    pCurrentSession->_currentConnection._urlBag._pAdvSearchKey = 0;
    pCurrentSession->_currentConnection._urlBag._pSongMetaDataKey = 0;


    pCurrentSession->_currentPreferences._autologin = 1;
    pCurrentSession->_currentPreferences._rememberpw = 0;
    pCurrentSession->_currentPreferences._pSongsDir = 0;
    pCurrentSession->_currentPreferences._pBaseURL = 0;

    return 0;
}

int fWriteSessionPrefs(struct uGroovesSession *pSession)
{
  int result = 0;
  int iFileDescriptor = 0;
  char *pPrefsFilename = 0;

  result = fGetPrefsFile(&pPrefsFilename);
  
  if (result < 0)
    {
      return -1;
    }


  return fWritePrefsToFile(pSession, pPrefsFilename);
}

int fDoSessionConnect(struct uGroovesSession *pSession)
{
    int result = 0;
    if(!pSession)
        return -1;
    
    if(pSession->_currentConnection._status == GROOVES_CSM_IDLE)
    {
        result = fChangeConnectionState(&pSession->_currentConnection,
                                        GROOVES_CSM_LINKING);
    }

    if(result < 0)
    {
        fChangeConnectionState(&pSession->_currentConnection,
                               GROOVES_CSM_TERMINATED);
    }

    return 0;
}

int fAttemptSessionLogin(struct uGroovesSession *pSession)
{
    if(!pSession)
        return -1;

    if((!pSession->_pUserName) ||
       (!pSession->_pPassWord))
    {
        fShowSessionLoginWindow(pSession);
        return -2;
    }

    return fDoSessionConnect(pSession);
}

int fDoQuit()
{
  struct uGroovesSession *pSession = 0;
  
  fGetSession(&pSession);

  GROOVES_ASSERT(pSession, "No Session while quitting!");
  
  fWriteSessionPrefs(pSession);

  /*See grooves.h / grooves.c*/
  fQuit();
}

int fAttemptSearch(struct uGroovesSession *pSession)
{
    if(pSession->_status != GROOVES_SSM_CONN)
    {
        fShowSessionLoginWindow(pSession);
        return 0;
    }

    return fAttemptConnSearch(&pSession->_currentConnection);
}
