#ifndef __STORE_LIB_H
#define __STORE_LIB_H

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "preferences.h"

/*Country data structs*/
struct _iStoreCountries
{
    char *m_pCountryName;
    char *m_pCountryCode;

    unsigned
     int   m_iCountryPos;
};

typedef struct _iStoreCountries iCountryEnt;

/*country struc operations*/ /*storelib_cc.c*/
int fGetCountryAtPos(unsigned int pos, iCountryEnt** pCountryEnt);
int fGetCountryByName(char *pName, iCountryEnt** pCountryEnt);
int fGetCountryByCode(char *pCode, iCountryEnt** pCountryEnt);

/*URL Consts*/ /*storelib.c*/
#define HTTP_CONST_EOL "\r\n"
extern char *HTTP_CONST_GET;
extern char *HTTP_CONST_PROTO;
extern char *HTTP_CONST_ENCOD;
extern char *HTTP_CONST_UAGEN;
extern char *HTTP_CONST_HOST;

extern char *HTTP_CONST_1AGNT;
extern char *HTTP_CONST_2AGNT;

extern int HTTP_FLAG_USE_1AGNT;
extern int HTTP_FLAG_USE_2AGNT;

#define HTTP_CONST_GETURL_ONESHOT -1

#define HTTP_CONST_TRANS_INC_UAGEN      1
#define HTTP_CONST_TRANS_INC_ACCLAN     2
#define HTTP_CONST_TRANS_INC_TIMEZ      4
#define HTTP_CONST_TRANS_INC_DSID       8
#define HTTP_CONST_TRANS_INC_XVALID     16
#define HTTP_CONST_TRANS_INC_ACCENC     32
#define HTTP_CONST_TRANS_INC_STOREFRONT 64
#define HTTP_CONST_TRANS_INC_HOST       128

#define HTTP_CONST_ACCLAN_TYP "Accept-Language:"
#define HTTP_CONST_ACCLAN_VAL "en-us, en;q=0.50"
#define HTTP_CONST_ACCENC_TYP "Accept-Encoding:"
#define HTTP_CONST_ACCENC_VAL "gzip, x-aes-cbc"
#define HTTP_CONST_TIMEZ_TYP "X-Apple-Tz:"
#define HTTP_CONST_DSID_TYP "X-Dsid:"
#define HTTP_CONST_XVALID_TYP "X-Apple-Validation:"
#define HTTP_CONST_STOREFRONT_TYP "X-Apple-Store-Front:"

/*URL Operations*/ /*storelib_url.c*/
int fGetURLSections(char *pURLToDownload,
                    char **ppProtoType,
                    char **ppServerAddr,
                    char **ppURI,
                    short *pPort);
int fGetURLServerInfo(char *pURLToDownload, char **ppServerAddr, short *pPort);

int fGetURLToFile(char *pURLToDownload, char *pFilename);
int fGetSockForURL(char *pURLToDownload, struct sockaddr_in *pServSock);
int fGetBufFromURL(int sock, char *pURLToDownload, char **ppBufToStore, int flags);

/*File Operations*/ /*storelib_file.c*/
int fOpenFileForWriting(char *pFilename, int flags);
int fOpenFileForReading(char *pFilename, int flags);
int fWriteBufToFile(char *pFilename, char *pBufToWrite, int len);

/*Base64 decode table*/
#define BASE64DECODETABLESIZE 123
extern char pBase64DecodeTable[BASE64DECODETABLESIZE];

/*conversion routines / util*/ /*storelib_util.c*/
int fDoUncompressBuffer(char *pUncompressStr,
                        char *pCompressedBuf,
                        int maxlen);

int fGetStrIndexOf(char *pHaystack, char *pString, int offset);

int fGetStrBetween(char **pStrToGet,
                   char *pBufToGrep,
                   int offset,
                   char *pStrStart,
                   char *pStrEnd);

int fConvertByteArrayFromBase64(char *pByteArray, int offset, 
                                int count, char **ppOutputBuf);
int fDoFromBase64(char *pByteArray, int offset, 
                  int count, char **ppOutputBuf);
int fDoFinalBase64TransformBlock(char *pByteArray, int offset, 
                                 int count, char **ppOutputBuf);
int fBlockCopy(char *pSrc, int srcOffset,
               char *pDest, int destOffset,
               int count);
int fDoFilterForWhitespaceChars(char *pBuffer, int offset, 
                                int count, char **ppFilterBuffer);
char fLookupBase64DecodeByte(char input);
int fStripWhitespace(char *pBufferToStrip);
int fGetNextLineFromBuf(const char *pBuffer, char *pLineBuf, 
                        int *pOffset, int size);
int fComputeValidator(char *pValidatorOut,
                      char *pURI,
                      char *pUserAgent,
                      int test);

/*iTunes Session/Connection stuff*/

struct uGroovesUrlBag
{
    char *_pSearchKey;
    char *_pSearchUrl;

    char *_pAdvSearchKey;
    char *_pAdvSearchUrl;
    
    char *_pSongMetaDataKey;
    char *_pSongMetaDataUrl;
    
    char *_pStoreFrontKey;
    char *_pStoreFrontUrl;
};

/*User Data Information*/
struct uGroovesConnection
{
    int   _status;     //part of a CSM
    int   _connectionSocket; //ptr to connection socket
    void *_pRelatedSession; //ptr to session
    char *_pUrlToFollow;    //url to follow to get to here
    int   _flags;           //flags to set when making requests
    struct uGroovesUrlBag _urlBag;
};

struct uGroovesSession
{
    char *_pUserName;
    char *_pPassWord;
    int   _attemptlogin;
    int   _status; //part of SSM

    struct GroovesPrefBlob    _currentPreferences;
    struct uGroovesConnection _currentConnection;
};

int fChangeConnectionState(struct uGroovesConnection *pConnection,
                           int nextState);
int fGetConnection(struct uGroovesConnection *pConnection);
int fInitConnectionToIdle(struct uGroovesConnection *pConnection);
int fAttemptConnSearch(struct uGroovesConnection *pConnection);

/*grooves connection states*/
#define GROOVES_CSM_IDLE       0
#define GROOVES_CSM_LINKING    1
#define GROOVES_CSM_AUTHITS    2
#define GROOVES_CSM_CONNECTED  3
#define GROOVES_CSM_TERMINATED 4

/*grooves session*/
int fChangeSessionState(struct uGroovesSession *pSession,
                        int nextState);
int fInitSessionToIdle(struct uGroovesSession *pCurrentSession);
int fGetSession(struct uGroovesSession **ppSession);
int fLoadSessionPreferences(struct uGroovesSession *pSession);
int fAttemptSessionLogin(struct uGroovesSession *pSession);
int fWriteSessionPrefs(struct uGroovesSession *pSession);
int fAttemptSearch(struct uGroovesSession *pSession);
int fDoQuit();

/*Grooves Session States*/

/*     [IDLE] <-------------,   Session starts out in idle state
 *       |                  |   It then gets passed to initialization routines
 *      \ /                 |   And goes to the initializing state (gathering
 * [INITIALIZING]           |    prefs information, etc.)
 *   |        |             |   Depending on what prefs are there, it will 
 *  \ /      \ /            |   either attempt a connection, or not connect.
 * [CONN]<=>[NOCONN]        |   It can go through multiple connection xitions.
 *             |            |   And can disconnect, and reconnect (thus the 2
 *            \ /           |   SM links).
 *       [TERMINATING]------'   Finally it goes to the no connection state, and
 *                              officially terminates. It only goes back to 
 *                              idle for completeness.
 */

#define GROOVES_SSM_IDLE   0 //idle state. Only when first created
#define GROOVES_SSM_INIT   1 //initialization state
#define GROOVES_SSM_NOCONN 2 //no connection
#define GROOVES_SSM_CONN   3 //connection state
#define GROOVES_SSM_TERM   4 //terminating state

/*HTTP related functions*/ /*storelib_http.c*/
int fOffsetOfHttpHeaders(char *pHttpBuf);
int fDoHTTPTransaction(int sock, char *pURLToDownload, char **ppBufToStore, 
                       int flags, struct uGroovesConnection *pConnection);
int fConstructHTTPGet(char *pURLToDownload, char **ppRequestBuf, 
                      int *len, int flags);

/*Test stuff / debug stuff*/
#define DEBUG_PRINT(s) printf s;
#define ASSERT_PRINT(s) printf s;
#define GROOVES_ASSERT(s, m) \
        grooves_assert(s, #s, m, 1, 0)

void
grooves_assert(int condition, char *pcondition,
               char *pmsg, int incl_bt, int crash);

int test_URL_section_get();
int test_Validator_code();

#endif /*__STORE_LIB_H*/
