/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "storelib.h"

iCountryEnt g_CountryEntries[22] =
{
    { "Australia", "143460", 0 },
    { "Austria", "143445-2", 1 },
    { "Belgium", "143446-2", 2 },
    { "Canada", "143455-6", 3 },
    { "Denmark", "143458", 4 },
    { "Finland", "143447", 5 },
    { "France", "143442", 6 },
    { "Germany", "143443", 7 },
    { "Great Britain", "143444", 8 },
    { "Greece", "143448", 9 },
    { "Ireland", "143449", 10 },
    { "Italy", "143450", 11 },
    { "Japan", "143462-1", 12 },
    { "Luxembourg", "143451-2", 13 },
    { "Netherlands", "143452", 14 },
    { "New Zealand", "143461", 15 },
    { "Norway", "143457", 16 },
    { "Portugal", "143453", 17 },
    { "Spain", "143454", 18 },
    { "Sweden", "143456", 19 },
    { "Switzerland", "143459", 20 },
    { "United States", "143441", 21 }
};

/*
 * fGetCountryAtPos
 * Description: Return the country entry from position, pos,
 *              and a result code indicating whether or not
 *              retrieval was a success
 * returns:     -1 on error, 0 on success and pCountryEnt will be valid
 * in:          unsigned int pos
 * out:         iCountryEnt* pCountryEnt, return code
 */

int fGetCountryAtPos(unsigned int pos, iCountryEnt** pCountryEnt)
{
    if(pos > 22)   /* MAX right now is 22...find a better way of doing this*/
        return -1;
    
    *pCountryEnt = &g_CountryEntries[pos];
    return 0;
}

/*
 * fGetCountryByName
 * Description: Based on pName, lookup the correct country entry
 *              and return it along with a result code indicating
 *              whether or not retrieval was a success
 * returns:     -1 on error, 0 on success and pCountryEnt will be valid
 * in:          char *pName
 * out:         iCountryEnt* pCountryEnt, return code
 */
int fGetCountryByName(char *pName, iCountryEnt** pCountryEnt)
{
    int curIdx = 0;

    if(pName == 0)
        return -1;
        
    while(strcmp(pName, g_CountryEntries[curIdx].m_pCountryName))
    {
        //indicates that this is NOT the one we want.
        curIdx++;
        if(curIdx > 22)
            return -1;
    }

    *pCountryEnt = &g_CountryEntries[curIdx];
    return 0;
}

/*
 * fGetCountryByCode
 * Description: Based on pCode, lookup the correct country entry
 *              and return it along with a result code indicating
 *              whether or not retrieval was a success
 * returns:     -1 on error, 0 on success and pCountryEnt will be valid
 * in:          char *pCode
 * out:         iCountryEnt* pCountryEnt, return code
 */
int fGetCountryByCode(char *pCode, iCountryEnt** pCountryEnt)
{
    int curIdx = 0;

    if(pCode == 0)
    {
        return -1;
    }

    while(strcmp(pCode, g_CountryEntries[curIdx].m_pCountryCode))
    {
        //indicates that this is NOT the one we want.
        curIdx++;
        if(curIdx > 22)
            return -1;
    }

    *pCountryEnt = &g_CountryEntries[curIdx];
    return 0;

}
