/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "storelib.h"
#include <string.h>
#include <sys/time.h>

int fGetLocationFromHTTPBuf(char *pURLToDownload, 
                            char *pHTTPBuf, char **ppNewURLToGo)
{
    char *pScratchBuf = 0;
    int   count     = 0;

    pScratchBuf = strstr(pHTTPBuf, "Location: ");

    if(!pScratchBuf)
    {
        printf("Location not found! [%s]\n",
               pHTTPBuf);
        return -1;
    }
    
    pScratchBuf += 10; /*go past 'Location: '*/
    
    while(pScratchBuf[count] != '\r')
        count++;
    
    *ppNewURLToGo = (char *)malloc(count+1);
    bzero(*ppNewURLToGo, count+1);
    
    memcpy(*ppNewURLToGo, pScratchBuf, count);

    return 0;
}

int fCheckForLocationChange(char *pURLToDownload, 
                            char *pHTTPBuf, char **ppNewURLToGo)
{

    int result = 0;
    int ret    = 0;

    if(pHTTPBuf == 0)
    {
        DEBUG_PRINT(("::fCheckForLocationChange called with null\n"));
        
        return 0;
    }

    /*Do we have a redirection?*/
    if((!strncmp("HTTP/1.0 302", pHTTPBuf, 12)) ||
       (!strncmp("HTTP/1.1 302", pHTTPBuf, 12)))
    {
        printf("[%s]\n302redirect\n", pHTTPBuf);
        result = fGetLocationFromHTTPBuf(pURLToDownload, 
                                         pHTTPBuf, ppNewURLToGo);
        ret = 1;
    }

    if(result)
    {
        DEBUG_PRINT(("::fCheckForLocationChange() getlocation failed!\n"));

        return -1;
    }
    
    return ret;
}

int fDoHTTPSimpleGet(int sock, char *pURLToDownload, char **ppBufToStore, 
                     int flags)
{
    //for now, just make this http only. https/ftp will be later..maybe
    //construct an HTTP Header
    char *pHTTPGetString   = 0;
    int   HTTPGetStringLen = 0;
    int currentAllocSize = 8192;
    int currentStreamPos = 0;
    int result = fConstructHTTPGet(
        pURLToDownload,
        &pHTTPGetString,
        &HTTPGetStringLen,
        flags);
    
    DEBUG_PRINT(("::entering fDoHTTPSimpleGet\n"));

    if(result < 0)
    {
        DEBUG_PRINT(("::fGetBufFromURL() = failed to construct string\n"));
        return -1;
    }
    
    result = write(sock, pHTTPGetString, strlen(pHTTPGetString));

    printf("wrote http get [%s]\n", pHTTPGetString);
    free(pHTTPGetString); //it's allocated, so free it


    if(result < 0)
    {
        DEBUG_PRINT(("ERROR TX BUFF!\n"));
        return -1;
    }

    //allocate 8192 bytes (8k..shouldn't be that much)
    *ppBufToStore = malloc(8192); 
    bzero(*ppBufToStore, 8192);

    printf("Getting to read..\n");
    result = read(sock, *ppBufToStore, 8192);
    if(result < 0)
    {
        free(*ppBufToStore);
        DEBUG_PRINT(("::fGetBufFromURL() = READ ERROR!!\n"));
        return -1;
    }

    currentStreamPos = result;

    if(result == 0)
    {
        //eof
        return -1;
    }

    while((result != 0) && !(result < 0))
    {
        printf("Iteration [%d][%d]!\n", currentAllocSize,currentStreamPos);
        *ppBufToStore = realloc(*ppBufToStore,
                                8192 + currentAllocSize);
        result = read(sock, *ppBufToStore + currentStreamPos, 
                      8192);
        currentAllocSize += 8192;
        currentStreamPos += result;
    }

    DEBUG_PRINT(("::exiting fGetBufFromURL\n"));
    return 0;    
}

int fHTTPCheckForGzipData(char *pHTTPBuf)
{
    char *pScratchBuf = 0;

    if(!pHTTPBuf)
        return -1;

    pScratchBuf = strstr(pHTTPBuf, "Content-Encoding: gzip");
    
    if(pScratchBuf)
    {
        return 1;
    }
    
    pScratchBuf = strstr(pHTTPBuf, "content-encoding: gzip");
    
    if(pScratchBuf)
        return 1;

    return 0;
}

/*
  This will decode a gzip'd http file and
  put the contents back into the http buf
*/
int fDoHTTPUnzipData(char **ppBufToUnzip)
{
    /*
      first get 2 ptrs...one that starts at gzip content
      and one that starts at non-gzip content.
    */
    char *pBufGzipStart = 0;
    char *pBufNewBuf    = 0;
    int   result        = 0;

    if(ppBufToUnzip == 0)
        return -1;

    if(*ppBufToUnzip == 0)
        return -1;

    pBufGzipStart = strstr(*ppBufToUnzip, "\r\n\r\n");
    pBufGzipStart += 4;

    pBufNewBuf = malloc(4*strlen(pBufGzipStart));
    result = fDoUncompressBuffer(pBufNewBuf,
                                 *ppBufToUnzip,
                                 4*strlen(pBufGzipStart));

    if(result < 0)
    {
        free(pBufNewBuf);
        return -1;
    }
    
    //For now, drop the original buffer
    free(*ppBufToUnzip);
    *ppBufToUnzip = pBufNewBuf;
    
    return 0;
}

int fDoHTTPTransaction(int sock, char *pURLToDownload, char **ppBufToStore, 
                       int flags, struct uGroovesConnection *pConnection)
{
    struct sockaddr_in localAddr, servAddr;
    char *pURLToForward = 0;
    char *pURItoIgnore = 0;
    char *pServAddr1 = 0;
    char *pServAddr2 = 0;
    short port = 0;
    char *pProto = 0;

    int result = fDoHTTPSimpleGet(sock,
                                  pURLToDownload,
                                  ppBufToStore, flags);

    DEBUG_PRINT(("::[%s] URLTOGET\n", pURLToDownload));

    if(result < 0)
    {
        return -1;
    }

    result = fCheckForLocationChange(pURLToDownload, *ppBufToStore, 
                                     &pURLToForward);
    
    if(result == 1)
    {
        free(*ppBufToStore);

        /*we might have been re-assigned! We need to make sure that if server
         *changes, we don't
         */
        result = fGetURLSections(pURLToDownload, &pProto, &pServAddr1,
                                 &pURItoIgnore, &port);
        
        if(result < 0)
            return -1;

        free(pURItoIgnore);

        result = fGetURLSections(pURLToForward, &pProto, &pServAddr2,
                                 &pURItoIgnore, &port);
        
        if(result < 0)
        {
            free(pServAddr1);
            return -1;
        }
        
        free(pURItoIgnore);

        if(strncmp(pServAddr1, pServAddr2, strlen(pServAddr1)))
        {
            free(pServAddr1);
            free(pServAddr2);
            if(pConnection)
            {
                close(pConnection->_connectionSocket);
                result = fGetSockForURL(pConnection->_pUrlToFollow,
                                        &servAddr);

                if(result < 0)
                {
                    return -1;
                }
                pConnection->_connectionSocket = result;

                localAddr.sin_family = AF_INET;
                localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
                localAddr.sin_port = htons(0);

                result = connect(pConnection->_connectionSocket,
                                 (struct sockaddr *) &servAddr, 
                                 sizeof(servAddr));

                if(result < 0)
                {
                    return -1;
                }
                
                result = fDoHTTPTransaction(pConnection->_connectionSocket,
                                         pURLToForward,
                                         &ppBufToStore,
                                         flags, pConnection);
                
            }
            else
            {
                //gzip shouldn't matter here...
                result =  fGetBufFromURL(HTTP_CONST_GETURL_ONESHOT,
                                         pURLToForward,
                                         &ppBufToStore,
                                         flags);
            }
            free(pURLToForward);
            return result;
        }

        free(pServAddr1);
        free(pServAddr2);

        result = fDoHTTPTransaction(sock, pURLToForward, ppBufToStore, 
                                    flags, 0);
        free(pURLToForward);
        return result;
    }else if(result < 0)
    {
        DEBUG_PRINT(("::fDoHTTPTransaction([%d] [%s] ...)"
                     " location check fail %d\n",
                     sock,
                     pURLToDownload,
                     result));
        return -1;
    } //if it's 0, don't forward
    
    if(flags & HTTP_CONST_TRANS_INC_ACCENC)
    {

        result = fHTTPCheckForGzipData(*ppBufToStore);

        DEBUG_PRINT(("Check for gzip yeilds: %d\n", result));
        if(result == 1)
        {
            result = fDoHTTPUnzipData(ppBufToStore);
            DEBUG_PRINT(("gunzip yeilds: %d\n", result));
        }
    }
    return 0;
}

int fConstructHTTPGet(char *pURLToDownload, char **ppRequestBuf, 
                      int *len, int flags)
{
    short portNum   = 0;
    struct timezone tz;
    struct timeval ts;
    char aTZ[20] = {0};
    char *pServName = 0;
    char *pURI      = 0;
    char *pProtocol = 0;
    char *pScratchStr = 0;
    int initialLen = 0;
    int incrLen = 0;
    char *pUserAgent = HTTP_CONST_1AGNT;
    int result = fGetURLSections(
        pURLToDownload,
        &pProtocol,
        &pServName,
        &pURI,
        &portNum);

    if(result < 0)
    {
        printf("::fConstructHTTPGet() = Error decoding URL\n");
        return -1;
    }

    initialLen += 1 +
        strlen(HTTP_CONST_GET) +
        strlen(pURI) +
        strlen(HTTP_CONST_PROTO) + 2 +
        strlen(HTTP_CONST_EOL);

    pScratchStr = (char *) malloc(1024); //alloc 1024 bytes upfront
    if(pScratchStr == 0)
    {
        free(pServName);
        free(pURI);

        printf("couldn't alloc buf: %d...\n", initialLen + 1);
        return -1;
    }
    bzero(pScratchStr, initialLen + 2);

    strncpy(pScratchStr, HTTP_CONST_GET, strlen(HTTP_CONST_GET));
    strncat(pScratchStr, " /", 2);
    strncat(pScratchStr, pURI, strlen(pURI));
    strncat(pScratchStr, " ", 1);
    strncat(pScratchStr, HTTP_CONST_PROTO, strlen(HTTP_CONST_PROTO));
    strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));
    
    /*We've used URI. Free it if we don't need the validator*/
    if(!(flags & HTTP_CONST_TRANS_INC_XVALID))
        free(pURI);

    DEBUG_PRINT(("[%s]\n", pScratchStr));
    if(flags & HTTP_CONST_TRANS_INC_ACCLAN)
    {
        incrLen = 1 +
            strlen(HTTP_CONST_ACCLAN_TYP) + 1 +
            strlen(HTTP_CONST_ACCLAN_VAL) +
            strlen(HTTP_CONST_EOL);

//        pScratchStr = (char *) realloc(pScratchStr, initialLen + incrLen);
        
        strncat(pScratchStr, HTTP_CONST_ACCLAN_TYP, 
                strlen(HTTP_CONST_ACCLAN_TYP));
        strncat(pScratchStr, " ", 1);
        strncat(pScratchStr, HTTP_CONST_ACCLAN_VAL,
                strlen(HTTP_CONST_ACCLAN_VAL));
        strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));

        initialLen += incrLen;
        //GROOVES_ASSERT((result >= 0), "Add AccLanguage failed");
    }
    
    if(flags & HTTP_CONST_TRANS_INC_TIMEZ)
    {
        gettimeofday(&ts, &tz);
        snprintf(aTZ,
                 20,
                 "%d",
                 tz.tz_minuteswest * -60);
        incrLen = 1 +
            strlen(HTTP_CONST_TIMEZ_TYP) + 1 +
            strlen(aTZ) +
            strlen(HTTP_CONST_EOL);

//        pScratchStr = (char *) realloc(pScratchStr, initialLen + incrLen);
        strncat(pScratchStr, HTTP_CONST_TIMEZ_TYP, 
                strlen(HTTP_CONST_TIMEZ_TYP));
        strncat(pScratchStr, " ", 1);
        strncat(pScratchStr, aTZ,
                strlen(aTZ));
        strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));


        initialLen += incrLen;
        //GROOVES_ASSERT((result >= 0), "Add Timezone failed");
        
    }
    
    if(flags & HTTP_CONST_TRANS_INC_UAGEN)
    {
        incrLen = 1 +
            strlen(HTTP_CONST_UAGEN) + 1 +
            strlen(HTTP_CONST_2AGNT) +
            strlen(HTTP_CONST_EOL);

//        pScratchStr = (char *) realloc(pScratchStr, initialLen + incrLen);
        strncat(pScratchStr, HTTP_CONST_UAGEN, 
                strlen(HTTP_CONST_UAGEN));
        strncat(pScratchStr, " ", 1);
        strncat(pScratchStr, HTTP_CONST_2AGNT,
                strlen(HTTP_CONST_2AGNT));
        strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));

        initialLen += incrLen;
        //GROOVES_ASSERT((result >= 0), "Add User Agent failed");
    }

    if(flags & HTTP_CONST_TRANS_INC_DSID)
    {
        incrLen = 1 +
            strlen(HTTP_CONST_DSID_TYP) + 1 +
            strlen("143441") +
            strlen(HTTP_CONST_EOL);

//        pScratchStr = (char *) realloc(pScratchStr, initialLen + incrLen);
        strncat(pScratchStr, HTTP_CONST_DSID_TYP, 
                strlen(HTTP_CONST_DSID_TYP));
        strncat(pScratchStr, " ", 1);
        strncat(pScratchStr, "143441",
                strlen("143441"));
        strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));

        initialLen += incrLen;
        //GROOVES_ASSERT((result >= 0), "Add DSID failed");
    }

    if(flags & HTTP_CONST_TRANS_INC_XVALID)
    {
        
        char pValidator[43] = "6B5C0D73-75C9B6E55BC3EE83859025E0E1B147E7";
        
        char *pUriLastPart = 0;
        int scan = strlen(pURI) - 1;
        
        while(pURI[scan] != '/' && scan > 0)
            scan--;
        
        pUriLastPart = pURI + scan;
        
/*        result = fComputeValidator(pValidator, pUriLastPart, 
                                   HTTP_CONST_2AGNT, 0);
*/
        incrLen = 1 +
            strlen(HTTP_CONST_XVALID_TYP) + 1 +
            strlen(pValidator) +
            strlen(HTTP_CONST_EOL);

//        pScratchStr = (char *) realloc(pScratchStr, initialLen + incrLen);
        strncat(pScratchStr, HTTP_CONST_XVALID_TYP, 
                strlen(HTTP_CONST_XVALID_TYP));
        strncat(pScratchStr, " ", 1);
        strncat(pScratchStr, pValidator,
                strlen(pValidator));
        strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));


        initialLen += incrLen;
        //GROOVES_ASSERT((result >= 0), "Add Validator failed");
        free(pURI);
    }

    if(flags & HTTP_CONST_TRANS_INC_ACCENC)
    {
        incrLen = 1 + 
            strlen(HTTP_CONST_ACCENC_TYP) + 1 +
            strlen(HTTP_CONST_ACCENC_VAL) +
            strlen(HTTP_CONST_EOL);

        strncat(pScratchStr, HTTP_CONST_ACCENC_TYP,
                strlen(HTTP_CONST_ACCENC_TYP));
        strncat(pScratchStr, " ", 1);
        strncat(pScratchStr, HTTP_CONST_ACCENC_VAL,
                strlen(HTTP_CONST_ACCENC_VAL));
        strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));
        
        initialLen += incrLen;
    }

    if(flags & HTTP_CONST_TRANS_INC_STOREFRONT)
    {
        incrLen = 1 +
            strlen(HTTP_CONST_STOREFRONT_TYP) + 1 +
            strlen("143441") +
            strlen(HTTP_CONST_EOL);

//        pScratchStr = (char *) realloc(pScratchStr, initialLen + incrLen);
        strncat(pScratchStr, HTTP_CONST_STOREFRONT_TYP, 
                strlen(HTTP_CONST_STOREFRONT_TYP));
        strncat(pScratchStr, " ", 1);
        strncat(pScratchStr, "143441",
                strlen("143441"));
        strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));

        initialLen += incrLen;
        //GROOVES_ASSERT((result >= 0), "Add Storefront failed");
    }

    if(flags & HTTP_CONST_TRANS_INC_HOST)
    {
        incrLen = 1 +
            strlen(HTTP_CONST_HOST) + 1 +
            strlen(pServName) +
            strlen(HTTP_CONST_EOL);

//        pScratchStr = (char *) realloc(pScratchStr, initialLen + incrLen);
        strncat(pScratchStr, HTTP_CONST_HOST, 
                strlen(HTTP_CONST_HOST));
        strncat(pScratchStr, " ", 1);
        strncat(pScratchStr, pServName,
                strlen(pServName));
        strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));


        initialLen += incrLen;
        free( pServName );
        pServName = 0;
        GROOVES_ASSERT((result >= 0), "Add Hostname failed");
    }

    incrLen = 2;
//    pScratchStr = (char *) realloc(pScratchStr, initialLen + incrLen);
    strncat(pScratchStr, HTTP_CONST_EOL, strlen(HTTP_CONST_EOL));

    *ppRequestBuf = pScratchStr;


    if(pServName)
        free(pServName);

    if(result < 0)
    {
        free(pScratchStr); //free it on error
        printf("::fConstructHTTPGet() = sprintf Failed %d\n", result);
        return -1;
    }
    
    return 0;
}

int fOffsetOfHttpHeaders(char *pHttpBuf)
{
    if(!pHttpBuf)
        return -1;

    return strstr(pHttpBuf, "\r\n\r\n");
}
