/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "storelib.h"
#include <openssl/md5.h>

int
test_URL_section_get()
{
    short portNum     = 0;
    char *pServName = 0;
    char *pURI      = 0;
    char *pProtocol = 0;
    char *pURLStringToDecode = 
        "http://www.testthisout.com:90/index.html\0";

    char *pURLStringToDecode2 = 
        "http://www.testthisagain.com/index2.html\0";

    
    int urlReturns = fGetURLSections(
        pURLStringToDecode,
        &pProtocol,
        &pServName,
        &pURI,
        &portNum);
    
    if((urlReturns < 0) ||
       (portNum != 90) ||
       (strncmp(pServName, "www.testthisout.com",
                strlen("www.testthisout.com"))) ||
       (strncmp(pProtocol, "http",
                4)) ||
       (strncmp(pURI, "index.html",
                10)) 
        )
    {
        printf("TESTFAILED: URL Section Get\n");
        printf("[%s]://[%s]:[%d]/[%s]\n",
               pProtocol,
               pServName,
               portNum,
               pURI);
        return -1;
    } else
    {
        printf("TESTPASSED!!!\n");
    }
    
    free(pServName);
    free(pURI);


    urlReturns = fGetURLSections(
        pURLStringToDecode2,
        &pProtocol,
        &pServName,
        &pURI,
        &portNum);
    
    if((urlReturns < 0) ||
       (portNum != 80) ||
       (strncmp(pServName, "www.testthisagain.com",
                strlen("www.testthisagain.com"))) ||
       (strncmp(pProtocol, "http",
                4)) ||
       (strncmp(pURI, "index2.html",
                11)) 
        )
    {
        printf("TESTFAILED: URL Section Get\n");
        printf("[%s]://[%s]:[%d]/[%s]\n",
               pProtocol,
               pServName,
               portNum,
               pURI);
        return -1;
    } else
    {
        printf("TESTPASSED!!!\n");
    }
    
    free(pServName);
    free(pURI);
    
}

int
test_base64_convert()
{
    char *pBase64String = "asdfASDF";
    char *pFinalBlockString = 0;
    
    int result = fConvertByteArrayFromBase64(
        pBase64String,
        0,
        strlen(pBase64String),
        &pFinalBlockString);

    if(result < 0)
    {
        printf("TESTFAILED: base64 decode block\n");
        return -1;
    }
    else
    {
        printf("TESTPASSED: [%s] became [%d]\n",
               pBase64String,
               pFinalBlockString);
        free(pFinalBlockString);
    }
    return 0;
}

int
test_Validator_code()
{
    char validatorArea[43] = {0};
    
    char *pURI = "search?term=311";

    char *pRandChar = 0;
    char *pValidateAgainst = 0;
    
    int randVal = 0;
    int result = fComputeValidator(validatorArea,
                                   pURI,
                                   HTTP_CONST_2AGNT, 0);
    printf("result: [%d] [%s]\n",
           result,
           validatorArea);
    
    if(result != 0)
        return -1;

    result = fComputeValidator(validatorArea,
                               pURI,
                               HTTP_CONST_2AGNT, 1);

    printf("result: [%d] [%s]\n",
           result,
           validatorArea);

    return 0;
}
