/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "storelib.h"

int fGetURLSections(char *pURLToDownload,
                    char **ppProtoType,
                    char **ppServerAddr,
                    char **ppURI,
                    short *pPort)
{
    int comparator = 0;
    char *pAdvanceStr = 0;
    char *pScratchStr = 0;
    int scratchInt = 0;
    int i = 10000;

    if(pURLToDownload == 0)
    {
        return -1;
    }

    comparator = strncmp(pURLToDownload, "http://", 7);
    if(comparator)
    {
        return -2;
    }
    *ppProtoType = "http";

    pAdvanceStr = pURLToDownload + 7;
    
    //just go until either : or /

    while((*pAdvanceStr != ':') &&
          (*pAdvanceStr != '/'))
    {
        pAdvanceStr++;
        scratchInt++;
    }

    *ppServerAddr = malloc(scratchInt+1);
    bzero(*ppServerAddr, scratchInt+1);
    memcpy(*ppServerAddr, pURLToDownload + 7,
           scratchInt);
    comparator = scratchInt;

    //yar, there be ports ahead
    if(*pAdvanceStr == ':')
    {
        pAdvanceStr++;
        scratchInt = 0;
        while(*pAdvanceStr != '/')
        {
            pAdvanceStr++;
            scratchInt++;
        }
        pScratchStr = malloc(scratchInt);
        bzero(pScratchStr, scratchInt);
        memcpy(pScratchStr, pURLToDownload +
               6 + comparator + scratchInt,
               scratchInt);
        *pPort = atoi(pScratchStr);
        free(pScratchStr);
        pScratchStr = 0;
        comparator += scratchInt + 2;
    } else
    {
        *pPort = 80;
        comparator += 1;
    }

    scratchInt = 0;
    pAdvanceStr++;
    while(*pAdvanceStr != '\0')
    {
        pAdvanceStr++;
        scratchInt++;
    }

    *ppURI = malloc(scratchInt+1);
    bzero(*ppURI, scratchInt+1);
    memcpy(*ppURI, pURLToDownload + 7 + comparator,
           scratchInt);

    return 0;
}
int fGetURLServerInfo(char *pURLToDownload, char **ppServerAddr, short *pPort)
{
    char *pReqURI = 0;
    char *pProtoType = 0;

    if(pPort == 0)
    {
        return -1;
    }
    
    if(fGetURLSections(pURLToDownload,
                       &pProtoType,
                       ppServerAddr,
                       &pReqURI,
                       pPort) != 0)

    {
        DEBUG_PRINT(("::fGetURLServerInfo([%s] ...) = %d\n",
               pURLToDownload ? pURLToDownload : "NULL", 
               -1));
        
        return -1;
    }

    free(pReqURI); //free the URI
    //we'd free the proto type as well, but it's not malloc'd

    return 0;
}


int fGetSockForURL(char *pURLToDownload, struct sockaddr_in *pServSock)
{
    struct hostent *pHostEntry = 0;
    char *pURLServer = 0;
    int sd = 0;
    short port = 0;
    int result = fGetURLServerInfo(pURLToDownload,
                                   &pURLServer,
                                   &port);

    if(result != 0)
    {
        DEBUG_PRINT(("::fGetSockForURL([%s], [%s]) = %d\n",
               pURLToDownload ? pURLToDownload : "NULL", 
               pServSock ? "SocketStruct" : "NULL",
               result));
        
        return -1;
    }

    pHostEntry = gethostbyname(pURLServer);
    if(!pHostEntry)
    {
        DEBUG_PRINT(("::fGetSockForURL([%s], [%s]) - pHostEntry failure: %s\n",
               pURLToDownload ? pURLToDownload : "NULL", 
               pServSock ? "SocketStruct" : "NULL",
               pURLServer,
               result));

        free(pURLServer);

        return -1;
    }

    sd = socket(AF_INET, SOCK_STREAM, 0);
    pServSock->sin_family = pHostEntry->h_addrtype;
    memcpy((char *) &pServSock->sin_addr.s_addr, 
           pHostEntry->h_addr_list[0], pHostEntry->h_length);
    pServSock->sin_port = htons(port);

    free(pURLServer);

    return sd;

}

int fGetBufFromURL(int sock, char *pURLToDownload, char **ppBufToStore, int flags)
{
    int result = 0;
    int bSetOneShot = 0;
    struct sockaddr_in localAddr, servAddr;
    if(sock == HTTP_CONST_GETURL_ONESHOT)
    {
        sock = fGetSockForURL(pURLToDownload, &servAddr);
        bSetOneShot = 1;
    }

    if(sock < 0)
    {
        DEBUG_PRINT(("::fGetBufFromURL([%d] [%s] ..)\n",
                     sock,
                     pURLToDownload ? pURLToDownload : "NULL"));
        
        return -1;        
    }

    if(bSetOneShot)
    {
        localAddr.sin_family = AF_INET;
        localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
        localAddr.sin_port = htons(0);

//    servAddr.sin_port = (short)htons(80);

        result = connect(sock, (struct sockaddr *) &servAddr, 
                         sizeof(servAddr));
        
        if(result < 0)
        {
            DEBUG_PRINT(("::fGetBufFromURL([%d] [%s] ..) conn = %d\n",
                         sock,
                         pURLToDownload ? pURLToDownload : "NULL",
                         result));
        
            close(sock);
            return -1;
            
        }
    }

    result = fDoHTTPTransaction(sock, pURLToDownload, ppBufToStore, flags, 0);

    if(bSetOneShot)
    {
        close(sock);
    }

    return result;
}


int fGetURLToFile(char *pURLToDownload, char *pFilename)
{
    struct sockaddr_in localAddr, servAddr;
    char *pBufToDownload = 0;
    int result = 0;
    int sock = fGetSockForURL(pURLToDownload, &servAddr);
    
    if (sock < 0)
    {
        DEBUG_PRINT(("::fGetURLToFile([%s], [%s]) = %d\n",
                    pURLToDownload ? pURLToDownload : "NULL", 
                    pFilename ? pFilename : "NULL",
                    sock));
        
        return -1;
    }

    localAddr.sin_family = AF_INET;
    localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    localAddr.sin_port = htons(0);

//    servAddr.sin_port = (short)htons(80);

    result = connect(sock, (struct sockaddr *) &servAddr, 
                     sizeof(servAddr));

    if(result < 0)
    {
        DEBUG_PRINT(("::fGetURLToFile([%s], [%s]) connect = %d [%X:%d]\n",
                    pURLToDownload ? pURLToDownload : "NULL", 
                    pFilename ? pFilename : "NULL",
                    result,
                    servAddr.sin_addr.s_addr,
                    (short)servAddr.sin_port));
        perror("connect");
        close(sock);
        return -1;
    }

    result = fGetBufFromURL(sock, pURLToDownload, &pBufToDownload, 
                            HTTP_CONST_TRANS_INC_UAGEN | 
                            HTTP_CONST_TRANS_INC_HOST);
    if(result != 0)
    {
        DEBUG_PRINT(("::fGetURLToFile([%s], [%s]) fGetBufFromURL = %d\n",
                    pURLToDownload ? pURLToDownload : "NULL", 
                    pFilename ? pFilename : "NULL",
                    result));

        return -1;
    }

    result = fWriteBufToFile(pFilename, pBufToDownload, 
                             strlen(pBufToDownload));
    if(result != 0)
    {
        DEBUG_PRINT(("::fGetURLToFile([%s], [%s]) fWriteBufToFile = %d\n",
                    pURLToDownload ? pURLToDownload : "NULL", 
                    pFilename ? pFilename : "NULL",
                    result));

        return -1;
    }
    
    if(pBufToDownload)
        free(pBufToDownload);

    close(sock);
    return 0;
}
