/**********************************************************
 * Grooves: Gtk+ iTunes Music Store client
 **********************************************************
 * By: Aaron Conole <conole@comcast.net>
 * Copyright (C) 2005 Aaron Conole
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "storelib.h"
#include <execinfo.h>
#include <zlib.h>
#include <string.h>
#include <openssl/md5.h>

char char2sixbit(unsigned char c)
{
    int  index = 0;
    char lookuptable[65] = 
        {"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"};

    if(c == '=')
        return 0;

    for(index = 0;
        index < 64;
        index++)
        if(lookuptable[index] == c)
            return index;
    return 0;
}

int fConvertByteArrayFromBase64(char *pByteArray, int offset, 
                                int count, char **ppOutputBuf)
{
    int  index         = 0;
    int  nPadCnt       = 0;
    int  nSourceLength = count;
    int  blockCount    = nSourceLength / 4;;
    int  nDecPadLength = blockCount * 3;
    int  nFinalLength  = 0;

    char *pSourceBits  = pByteArray + offset;
    char *pOnePassBuf  = malloc(count);
    char *pXFormBlock  = 0;
    char *pOutputBuf   = 0;

    char xformbyte1, xformbyte2, xformbyte3, xformbyte4;
    char xformtemp1, xformtemp2, xformtemp3, xformtemp4;
    
    if(!pOnePassBuf)
        return -1;

    for(index = 0;
        index < 2;
        index++)
        if(pSourceBits[nSourceLength - index - 1] == '=')
            nPadCnt++;

    nFinalLength = nDecPadLength - nPadCnt;

    for(index = 0;
        index < nSourceLength;
        index++)
        pOnePassBuf[index] = char2sixbit(pSourceBits[index]);
    
    pXFormBlock = malloc(nDecPadLength);
    if(!pXFormBlock)
    {
        free(pOnePassBuf);
        return -1;
    }

    for (index = 0;
         index < blockCount;
         index++)
    {
        xformtemp1 = pOnePassBuf[index * 4];
        xformtemp2 = pOnePassBuf[index * 4+1];
        xformtemp3 = pOnePassBuf[index * 4+2];
        xformtemp4 = pOnePassBuf[index * 4+3];

        xformbyte1 =(char) xformtemp1 << 2;
        xformbyte2 =(char) (xformtemp2 & 48) >> 4;
        xformbyte2 +=(char) xformbyte1;

        xformbyte1 =(char) (xformtemp2 & 15) << 4;
        xformbyte3 =(char) (xformtemp3 & 60) >> 2;
        xformbyte3 +=(char) xformbyte1;
        
        xformbyte1 =(char) (xformtemp3 & 3) <<  6;
        xformbyte4 =(char) xformtemp4;
        xformbyte4 +=(char) xformbyte1;

        pXFormBlock[index*3]   = (char)xformbyte2;
        pXFormBlock[index*3+1] = (char)xformbyte3;
        pXFormBlock[index*3+2] = (char)xformbyte4;
    }

    pOutputBuf = malloc(nFinalLength+1);
    if(pOutputBuf == 0)
    {
        free(pOnePassBuf);
        free(pXFormBlock);
        return -1;
    }

    bzero(pOutputBuf, nFinalLength+1);
    for(index = 0;
        index < nFinalLength;
        index++)
    {
        pOutputBuf[index] = pXFormBlock[index];
    }
    *ppOutputBuf = pOutputBuf;
    free(pOnePassBuf);
    free(pXFormBlock);

    return 0;
}

/**
 * Whitespace characters are:
 *   ' '  blank
 *   '\n' newline
 *   '\t' tab
 *   '\f' formfeed
 *   '\r' carriage return
 */
int fIsWhitespaceChar(char b)
{
    
    if(b == '\n' ||
       b == '\t' ||
       b == '\r' ||
       b == '\f' ||
       b == ' '  )
    {
        return b;
    }

    return 0;
}

int fDoFilterForWhitespaceChars(char *pBuffer, int offset, 
                                int count, char **ppFilterBuffer)
{
    int end = offset + count;
    int len = 1;
    int ptr = 0;
    int i   = 0;
    char b  = 0;
    char *pFilter = 0;

    pFilter = (char *)malloc(1);
    if(pFilter == 0)
    {
        DEBUG_PRINT((
            "::fDoFilterForChars([%s] [%d] [%d] ...) MALLOC() fail\n",
            pBuffer, offset,
            count));

        return -1;
    }
    
    for (i = offset; i < end; i++) {
        if (!fIsWhitespaceChar(b)) {
            if (ptr >= len) {
                len <<= 1;
                pFilter = (char *)realloc(pFilter, len);
            }
            pFilter [ptr++] = b;
            b = pBuffer[ptr];    
        }
    }

    *ppFilterBuffer = pFilter;
    return ptr;

}

int fBlockCopy(char *pSrc, int srcOffset,
               char *pDest, int destOffset,
               int count)
{
    char *pDst = pDest + destOffset;
    char *s    = pSrc + srcOffset;
    memcpy(pDst, s, count);
    return 0;
}

int fStripWhitespace(char *pBufferToStrip)
{
    int x,y;

    x = y = 0;
    
    while(pBufferToStrip[x] != '\0')
    {
        if(!fIsWhitespaceChar(pBufferToStrip[x]))
        {
            /*not whitespace, time to keep this one*/
            pBufferToStrip[y] = pBufferToStrip[x];
            y++;
        }
        x++;
    }

    pBufferToStrip[y] = '\0';

    return y;
}

int fGetNextLineFromBuf(const char *pBuffer, char *pLineBuf, 
                        int *pOffset, int size)
{
    char c = 0;
    int count = 0;
    int done = 0;
    int offset = *pOffset;

    while((!done) && 
          (count < size))
    {
        c = pBuffer[count + offset];

        if(c == 0)
        {
            if(count == 0)
                return 0;
            else
                done = 1;
        }

        *pLineBuf = c;
        pLineBuf++;
        count++;

        if(c == '\n')
            done = 1;
    }
    
    *pLineBuf = 0;
    *pOffset = offset + count;
    
    return count;
}

int fDoUncompressBuffer(char *pUncompressStr,
                        char *pCompressedBuf,
                        int maxlen)
{
    return uncompress(pUncompressStr, &maxlen, pCompressedBuf, 
                      strlen(pCompressedBuf));
}

int fGetStrIndexOf(char *pHaystack, char *pString, int offset)
{
  register char *pS = pHaystack;
  register char *p = pHaystack + offset;
  register int tlen = strlen(pString);
  
  for(; (p = strchr(p, *pString)) != 0; p++)
    {
      if(strncmp(p, pString, tlen) == 0)
	{
	  return p - pS;
	}
    }

  return -1;
}

int fGetStrBetween(char **pStrToGet,
                   char *pBufToGrep,
                   int offset,
                   char *pStrStart,
                   char *pStrEnd)
{
    int startLoc = 0;
    int endLoc = 0;
    
    startLoc = 
        fGetStrIndexOf(pBufToGrep, pStrStart, offset) +
        strlen(pStrStart);

    endLoc =
        fGetStrIndexOf(pBufToGrep, pStrEnd, offset);

    if((startLoc < 0) ||
       (endLoc < 0))
    {
        return -1;
    }
       
    *pStrToGet = malloc(1 + endLoc - startLoc);

    bzero(*pStrToGet, (1 + endLoc - startLoc));
    strncpy(*pStrToGet,
            pBufToGrep + startLoc,
            endLoc - startLoc);

    DEBUG_PRINT(("::fGetStringBetween() [%s]<-[%s]->[%s] [%d] [%d]:[%d]\n",
		 pStrStart, *pStrToGet, pStrEnd,
		 offset, startLoc, endLoc));
    return 0;
}

int fDoAsciiByteConvert(char *pBytes,
                        int  nBytesIn,
                        int  nInOffset,
                        char *pOutBytes,
                        int  nOutOffset)
{
    char c = 0;
    int start = nBytesIn;
    while(nBytesIn--)
    {
        c = pBytes[nInOffset++];
        if(c < 0x80)
            pOutBytes[nOutOffset++] = c;
        else
            pOutBytes[nOutOffset++] = '?';
    }
    
    return start;
}

int fComputeValidator(char *pValidatorOut,
                      char *pURI,
                      char *pUserAgent,
                      int  test)
{
    int randValid = 0;
    char aPart1Validator[9] = {0};
    char aPart2Validator[43] = {0};

    char aPart1AsciiValid[9] = {0};
    char aPart2AsciiValid[43] = {0};
    char *pUABytes = 0;

//    char  aStaticRandFrom64[26] = /*{"a1d2c33f4b2cc4d21fd4fbb521"}*/
//        "ROkjAaKid4EUF5kGtTNn3Q==";
    char  aStaticRandDecoded[17] = {0x47, 0x49, 0x23, 0x01, 
                                    0xA2, 0xA2, 0x77, 0x81, 
                                    0x14, 0x17, 0x99, 0x06, 
                                    0xB5, 0x33, 0x67, 0xDD, 0x00};
    char  aHoldStr[3]       = {0};
    char *pReturnFromRand64 = 0;
    int   index             = 0;
    int   result            = 0;

    char *pUserURI          = pURI;

    if((pURI == 0) ||
       (pUserAgent == 0) ||
       (pValidatorOut == 0))
       return -1;

    if(!test)
    {
        srand((unsigned)time(0));
        
        randValid = rand();
        
        snprintf(aPart1Validator,
                 9,
                 "%04X%04X",
                 rand() % 65535,
                 rand() % 65535);
    }
    else
    {
        strcpy(aPart1Validator, "6B5C0D73");
        pUserURI = "/search?term=311";
    }
//    result = fConvertByteArrayFromBase64(aStaticRandFrom64, 0,
//                                         24, &pReturnFromRand64);
    
    if(result < 0)
        return -1;

    //now compute MD5 stuffs
    MD5_CTX context;
    
    MD5_Init(&context);
    MD5_Update(&context, pUserURI, strlen(pUserURI));
    MD5_Update(&context, pUserAgent, strlen(pUserAgent));
    MD5_Update(&context, aStaticRandDecoded, strlen(aStaticRandDecoded));
    MD5_Update(&context, aPart1Validator, 8);
    MD5_Final(aPart2Validator, &context);

    free(pReturnFromRand64);
    //at this point we have both sides of the coin.
    snprintf(pValidatorOut,
             9,
             "%s",
             aPart1Validator);

    strcat(pValidatorOut, "-");

    for(index = 0;
        index < 16;
        index++)
    {
        snprintf(aHoldStr,
                 3,
                 "%02X",
                 aPart2Validator[index] & 0xff);
        strcat(pValidatorOut, aHoldStr);
    }

    if(test)
    {
        if(strncmp(pValidatorOut, "6B5C0D73-75C9B6E55BC3EE83859025E0E1B147E7",
                   strlen(pValidatorOut)))
            DEBUG_PRINT(("FAIL!!"));
    }

    return 0;
}

void
grooves_assert(int condition, char *pcondition,
               char *pmsg, int incl_bt, int crash)
{
    void *pBTArray[20];
    size_t size = 0;
    char **pStrings;
    size_t i;

    if(!condition)
    {
        ASSERT_PRINT((pmsg));
    } else
        return;
    
    if(incl_bt)
    {
        size = backtrace(pBTArray, 20);
        pStrings = backtrace_symbols (pBTArray, size);
        ASSERT_PRINT(("Assert %s Failed\n",
                      pcondition));
        ASSERT_PRINT(("catch trace with %d entries\n",
                      size));
        for(i = 0; i < size; i++)
            ASSERT_PRINT(("Entry %d: [%s]\n", size,
                          pStrings[i]));
        free(pStrings);
    }

    if(crash)
        exit (-1);
}
